using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseRepresentation;
using System.Collections;

namespace EnterpriseAnalysis
{
    class ComponentAnalyzer
    {
        ArrayList result;
        EnterpriseRepresentation.EnterpriseRepresentation ent;
        public ComponentAnalyzer(){}

       
        public ArrayList GetComponentAnalysis(EnterpriseRepresentation.EnterpriseRepresentation ent)
        {
            this.ent = ent;
            result = new ArrayList();
            ArrayList systems = ent.GetReferenceToAllSystems();
                        
            foreach(EnterpriseRepresentation.System sys in systems)
            {
                if (sys.components != null)
                {
                    ParseComponent(sys.components);
                }
            }
            
            return result;
        } 

        public void ParseComponent(ArrayList c)
        {
            foreach (Component comp in c)
            {
                double crit;
                double critsum = 0.0;
                double availibility = 1.0;
                double baseavb;


                availibility *= GetAvailibilityOf(comp);
                baseavb = availibility;

                EnterpriseRepresentation.Environment env;
                object temp;

                temp = comp.parent;

                while (!temp.GetType().ToString().Equals("EnterpriseRepresentation.System"))
                {
                    temp = ((Component)(temp)).parent;
                    availibility *= GetAvailibilityOf((EnterpriseEntity)(temp));
                }

                env = ((EnterpriseRepresentation.System)(temp)).env;
                
                double envavail = 1.0;
                double avb;

                while (env.prev != null)
                {
                    avb = GetAvailibilityOf(env);                    
                    envavail *= avb;
                    env = env.prev;
                }

                availibility *= envavail;                

                if (comp.GotTagValue("parallel"))
                {                   
                   int num = 1;
                   num = int.Parse(comp.GetTagValue("parallel"));

                   availibility *= (1 - Math.Pow((1 - GetAvailibilityOf(comp)), num));
                }
                else
                {
                    availibility *= GetAvailibilityOf(comp);                   
                }


                ArrayList processes = ent.BusinessProcesses;               
                crit = 0.0;
                double sumimportance = 0.0;

                foreach (BusinessProcess bp in processes)
                {
                  
                    double prob = GetAvgComponentProbability(comp, bp);

                    //For each Subsystem operation that interacts with the given component
                    foreach (SubSystemOperation subsysop in comp.subsysops)
                    {
                        if (IsCritical(subsysop) && GetProcessOf(subsysop).Equals(bp))
                        {                                                       
                            //Gets the criticality of the component based on criticality the process is realizes 
                            //and the probability that it is executed during the proccess.
                            crit = bp.Criticality * prob;  
                        }
                    }

                    critsum += crit;

                    double importance = 1.0;
                    //int count = 0;
                    
                    foreach (Component component in ent.GetReferanceToAllComponents())
                    {
                        if (ComponentCriticallyRealizesProcess(component, bp))
                        {
                            if (component.ID != comp.ID)
                            {
                                importance *= GetAvailibilityOf(component);
                                //count++;
                            }
                        }
                    }

                    //if(count != 0)
                    sumimportance += importance * crit;


                }

                EnterpriseRepresentation.EnterpriseRepresentation exent = ent;
                result.Add(new ComponentAnalysisResult(comp, critsum, availibility, baseavb, sumimportance));                
                ParseComponent(comp.components);
            }
        }


        public BusinessProcess GetProcessOf(SubSystemOperation subsys)
        {
            return subsys.parent.businessfunction.process;
        }

        public double GetAvgComponentProbability(EnterpriseRepresentation.Component comp, BusinessProcess bp)
        {
            //A list for storing business functions
            ArrayList bfs = new ArrayList();

            foreach (SubSystemOperation subsysop in comp.subsysops)
            {
                if (GetProcessOf(subsysop).Equals(bp))
                {
                    bool contains = false;

                    foreach (BusinessFunction bf in bfs)
                    {
                        if (bf.ID == subsysop.parent.businessfunction.ID)
                        {
                            contains = true;
                            break;
                        }
                    }

                    if (!contains)
                    {
                        bfs.Add(subsysop.parent.businessfunction);
                    }
                }
            }

            //Calculating how often the business processes are used in one execution of the Business Process.
            double sum = ent.GetProbabilityOfBusinessFunctions(bfs, bp.First, 1.0);

            //Returning the probability.
            return sum;
        }


        public bool ComponentCriticallyRealizesProcess(Component c, BusinessProcess bp)
        {

            foreach (EnterpriseRepresentation.SubSystemOperation op in c.subsysops)
            {
                BusinessProcess test = GetProcessOf(op);

                if (GetProcessOf(op).Equals(bp) && IsCritical(op))
                {
                    return true;
                }

            }

            return false;
        }

        public bool ComponentRealizesProcess(Component c, BusinessProcess bp)
        {

            foreach (EnterpriseRepresentation.SubSystemOperation op in c.subsysops)
            {
                BusinessProcess test = GetProcessOf(op);
               
                if(GetProcessOf(op).Equals(bp))
                {
                    return true;
                }

            }

            return false;
        }
            
        public bool IsCritical(SubSystemOperation subsysop)
        {

            bool critical = true;

            if (subsysop.HasStereoType("NonCritical"))
            {
                critical = false;
            }

            if (subsysop.parent.HasStereoType("NonCritical"))
            {
                critical = false;
            }

            if (subsysop.parent.businessfunction.HasStereoType("NonCritical"))
            {
                critical = false;
            }
            
            return critical;
        }

        public double GetAvailibilityOf(EnterpriseRepresentation.EnterpriseEntity entity)
        {
            double avb = 1.0;

            if (entity.GotTagValue("MTBF") && entity.GotTagValue("MTTR"))
            {
                try
                {
                    double failurerate = double.Parse(entity.GetTagValue("MTBF"));
                    double repairtime = double.Parse(entity.GetTagValue("MTTR"));

                    avb = (failurerate / (failurerate + repairtime));
                }
                catch
                {
                }
            }

            return avb;

        }


    }
}
