using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using EnterpriseRepresentation;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace EnterpriseAnalysis
{
    public partial class EnterpriseRepresentationCreator : UserControl
    {

        EnterpriseRepresentation.EnterpriseRepresentation entrep = new EnterpriseRepresentation.EnterpriseRepresentation();
        ImageList list;
        ImageList list2;
        public BusinessProcess currentbp;
        string StartUpPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);   

        public EnterpriseRepresentationCreator()
        {
            InitializeComponent();

            list = new ImageList();
            list.Images.Add(new Icon(StartUpPath + "\\None.ico"));
            list.Images.Add(new Icon(StartUpPath + "\\System.ico"));
            list.Images.Add(new Icon(StartUpPath + "\\Environment.ico"));
            list.Images.Add(new Icon(StartUpPath + "\\Component.ico"));

            entviewstruct.ImageList = list;

            list2 = new ImageList();
            list2.Images.Add(new Icon(StartUpPath + "\\None.ico"));
            list2.Images.Add(new Icon(StartUpPath + "\\Process.ico"));
            list2.Images.Add(new Icon(StartUpPath + "\\Function.ico"));
            list2.Images.Add(new Icon(StartUpPath + "\\Sysop.ico"));
            list2.Images.Add(new Icon(StartUpPath + "\\SubSysOp.ico"));

            entviewprocess.ImageList = list2;
          
        }



        public void PresentEnterpriseRepresentation()
        {
            entviewprocess.Nodes.Clear();
            entviewstruct.Nodes.Clear();

            foreach (BusinessProcess bp in entrep.BusinessProcesses)
            {
                TreeNode processnode = entviewprocess.Nodes.Add(bp.Name, bp.Name + " :: " + bp.Criticality, 1);
                processnode.Tag = bp;

                if (bp.First != null)
                {
                    ArrayList bfs = entrep.GetEachUniqueBusinessFunction(bp);

                    foreach (BusinessFunction bf in bfs)
                    {
                        TreeNode functionnode = processnode.Nodes.Add(bf.ID, bf.Name, 2);
                        functionnode.Tag = bf;

                        SystemOperation first = bf.FirstOperation;

                        if (first != null)
                        {

                            TreeNode sysopnode = functionnode.Nodes.Add(first.ID, first.Name, 3);
                            sysopnode.Tag = first;

                            SubSystemOperation sso = first.firstoperation;

                            if (sso != null)
                            {
                                sysopnode.Nodes.Add(sso.ID, sso.Name, 4);

                                while (sso.next != null)
                                {
                                    sso = sso.next;
                                    TreeNode subsysnode = sysopnode.Nodes.Add(sso.ID, sso.Name, 4);
                                    subsysnode.Tag = sso;
                                }

                            }




                            while (first.next != null)
                            {
                                first = first.next;
                                sysopnode = functionnode.Nodes.Add(first.ID, first.Name, 3);
                                sysopnode.Tag = first;

                                sso = first.firstoperation;

                                if (sso != null)
                                {
                                    sysopnode.Nodes.Add(sso.ID, sso.Name, 4);

                                    while (sso.next != null)
                                    {
                                        sso = sso.next;
                                        TreeNode subsysnode = sysopnode.Nodes.Add(sso.ID, sso.Name, 4);
                                        subsysnode.Tag = sso;
                                    }

                                }

                            }


                        }


                    }
                }

            }

            TreeNode treenodes = new TreeNode();

            if (entrep.Environment != null)
            {
                ParseEnvironments(entrep.Environment, treenodes);

                entviewstruct.Nodes.Add(treenodes.Nodes[0]);
            }

            entviewstruct.ExpandAll();
            entviewprocess.ExpandAll();

        }


        public void ParseEnvironments(EnterpriseRepresentation.Environment e, TreeNode treenode)
        {



            TreeNode next = treenode.Nodes.Add(e.ID, e.Name, 2);
            next.Tag = e;

            foreach (EnterpriseRepresentation.Environment ent in e.environments)
            {
                ParseEnvironments(ent, next);
            }

            foreach (EnterpriseRepresentation.System sys in e.systems)
            {
                TreeNode s = next.Nodes.Add(sys.ID, sys.Name, 1);
                s.Tag = sys;

                if (sys.components.Count > 0)
                {
                    AddComponents(sys.components, s);
                }
            }
        }

        private void addenv_Click(object sender, EventArgs e)
        {
            xmiopener.FileName = "";

            xmiopener.ShowDialog();

            if (xmiopener.FileName.Length > 0)
            {
                try
                {
                    entrep.AddEnvironments(xmiopener.FileName);
                }
                catch (Exception ev)
                {
                    MessageBox.Show(ev.Message);
                }

                PresentEnterpriseRepresentation();
            }
        }

        private void addcomps_Click(object sender, EventArgs e)
        {

            xmiopener.FileName = "";

            xmiopener.ShowDialog();

            foreach (string filename in xmiopener.FileNames)
            {
                if (filename.Length > 0)
                {
                    try
                    {
                        entrep.AddComponents(xmiopener.FileName);
                    }
                    catch (Exception ev)
                    {
                        MessageBox.Show(ev.Message);
                    }

                    PresentEnterpriseRepresentation();
                }
            }

        }

        public void AddComponents(ArrayList c, TreeNode node)
        {
            foreach (EnterpriseRepresentation.Component comp in c)
            {
                TreeNode next = node.Nodes.Add(comp.ID, comp.Name, 3);
                next.Tag = comp;
                AddComponents(comp.components, next);

            }


        }

        private void addfunction_Click(object sender, EventArgs e)
        {
            xmiopener.FileName = "";
            xmiopener.ShowDialog();

            if (xmiopener.FileName.Length > 0)
            {
                try
                {
                    if (processname.Text == "" && xmiopener.FileName.Contains(".xmi"))
                    {

                        string filename = xmiopener.FileName;
                        int indeks = filename.LastIndexOf("\\");
                        filename = filename.Remove(0, indeks+1);
                        filename = filename.Replace(".xmi", "");

                        entrep.AddProcess(filename, xmiopener.FileName, (int)processcrit.Value);
                    }
                    else
                    {
                        entrep.AddProcess(processname.Text, xmiopener.FileName, (int)processcrit.Value);
                    }
               }
                catch (Exception ev)
                {
                    MessageBox.Show(ev.Message);
                }

                PresentEnterpriseRepresentation();
            }

        }

        private void addsystemops_Click(object sender, EventArgs e)
        {

            xmiopener.FileName = "";
            xmiopener.ShowDialog();

            if (xmiopener.FileName.Length > 0)
            {
                if (entviewprocess.SelectedNode != null)
                {
                    if (entviewprocess.SelectedNode.Tag.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
                    {

                        TreeNode parent = entviewprocess.SelectedNode.Parent;

                        try
                        {
                            entrep.AddSystemOperations((((BusinessProcess)parent.Tag)).Name, ((BusinessFunction)(entviewprocess.SelectedNode.Tag)).ID, xmiopener.FileName);
                        }
                        catch (Exception ev)
                        {
                            MessageBox.Show(ev.Message);
                        }

                        PresentEnterpriseRepresentation();

                    }
                }
            }
        }

        private void addsubsysop_Click(object sender, EventArgs e)
        {

            xmiopener.FileName = "";
            xmiopener.ShowDialog();

            if (xmiopener.FileName.Length > 0)
            {
                if (entviewprocess.SelectedNode != null)
                {
                    if (entviewprocess.SelectedNode.Tag.GetType().ToString().Equals("EnterpriseRepresentation.SystemOperation"))
                    {
                        TreeNode parent = entviewprocess.SelectedNode.Parent;

                        try
                        {
                            entrep.AddSubSystemOperations((((BusinessProcess)parent.Parent.Tag)).Name, ((BusinessFunction)(entviewprocess.SelectedNode.Parent.Tag)).ID, ((SystemOperation)(entviewprocess.SelectedNode.Tag)).ID, xmiopener.FileName);
                        }
                        catch (Exception ev)
                        {
                            MessageBox.Show(ev.Message);
                        }

                        PresentEnterpriseRepresentation();

                    }
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            EnterpriseRepresentationCreator prev = this;

            try
            {
                ((MainMenu)(this.Parent)).SetUserControl(new AnalysisPresenter(entrep, prev));
            }
            catch
            {
                MessageBox.Show("Error analyzing the Enterprise. Check structure");
            }
        }

        public void SerializeEnterprise(string filename)
        {

            FileStream fStream = new FileStream(filename, FileMode.Create);
            BinaryFormatter binFormat = new BinaryFormatter();

            try
            {
                ArrayList arraylist = new ArrayList();
                arraylist.Add(entrep.Environment);
                arraylist.Add(entrep.BusinessProcesses);

                binFormat.Serialize(fStream, arraylist);
            }
            catch (Exception e)
            { MessageBox.Show(e.Message); }
            finally
            {
                fStream.Close();
            }

        }

        public void DeSerializeEnterprise(string filename)
        {
            FileStream fStream = null;

            if (File.Exists(filename))
            {
                try
                {

                    fStream = new FileStream(filename, FileMode.Open);
                    BinaryFormatter binFormat = new BinaryFormatter();
                    ArrayList arraylist = (ArrayList)binFormat.Deserialize(fStream);

                    entrep.Environment = (EnterpriseRepresentation.Environment)(arraylist[0]);
                    entrep.BusinessProcesses = (ArrayList)(arraylist[1]);
                }
                catch (Exception e) { MessageBox.Show(e.Message); }
                finally
                {
                    fStream.Close();
                }
            }
        }

        private void save_Click(object sender, EventArgs e)
        {
            entsaver.ShowDialog();

            if (entsaver.FileName.Length > 0)
            {
                SerializeEnterprise(entsaver.FileName);
            }
        }

        private void loader_Click(object sender, EventArgs e)
        {
            entopener.ShowDialog();

            if (entopener.FileName.Length > 0)
            {
                DeSerializeEnterprise(entopener.FileName);
            }

            PresentEnterpriseRepresentation();

        }

        private void entviewprocess_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                DialogResult dr = new DialogResult();
                dr = MessageBox.Show(this, "Do you want to delete the selected item?", "", MessageBoxButtons.YesNo);

                if (dr.ToString().Equals("Yes"))
                {
                    TreeNode node = entviewprocess.SelectedNode;
                    string id = "";

                    if (node.Tag.GetType().ToString().Equals("EnterpriseRepresentation.BusinessProcess"))
                    {
                        id = ((EnterpriseRepresentation.BusinessProcess)(node.Tag)).Name;

                        for (int i = 0; i < entrep.BusinessProcesses.Count; i++)
                        {
                            if (((BusinessProcess)(entrep.BusinessProcesses[i])).Name.Equals(id))
                            {
                                entrep.BusinessProcesses.RemoveAt(i);
                            }
                        }
                    }

                    else if (node.Tag.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
                    {
                        id = ((EnterpriseRepresentation.BusinessFunction)(node.Tag)).ID;
                        entrep.RemoveSystemFunction(((BusinessProcess)(node.Parent.Tag)).Name, id);
                    }

                    else if (node.Tag.GetType().ToString().Equals("EnterpriseRepresentation.SystemOperation"))
                    {
                        id = ((EnterpriseRepresentation.SystemOperation)(node.Tag)).ID;

                        entrep.RemoveSubSystemOperations(((BusinessFunction)(node.Parent.Tag)).ID, id, ((BusinessProcess)(node.Parent.Parent.Tag)).Name);
                    }

                    this.PresentEnterpriseRepresentation();
                }


            }
        }

        private void entviewstruct_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                DialogResult dr = new DialogResult();
                dr = MessageBox.Show(this, "Do you want to delete the selected item?", "", MessageBoxButtons.YesNo);

                if (dr.ToString().Equals("Yes"))
                {
                    TreeNode node = this.entviewstruct.SelectedNode;
                    string id = "";

                    if (node.Tag.GetType().ToString().Equals("EnterpriseRepresentation.Component"))
                    {
                        id = ((EnterpriseRepresentation.Component)(node.Tag)).ID;


                        foreach (EnterpriseRepresentation.Component comp in entrep.GetReferanceToAllComponents())
                        {

                            if (comp.ID.Equals(id))
                            {
                                comp.components.Clear();
                            }

                        }


                    }
                    else if (node.Tag.GetType().ToString().Equals("EnterpriseRepresentation.Environment"))
                    {
                        id = ((EnterpriseRepresentation.Environment)(node.Tag)).ID;


                        foreach (EnterpriseRepresentation.Environment env in entrep.GetReferenceToAllEnvironments())
                        {
                            if (env.ID.Equals(id))
                            {
                                env.environments.Clear();
                                env.systems.Clear();
                            }
                        }
                    }
                    else if (node.Tag.GetType().ToString().Equals("EnterpriseRepresentation.System"))
                    {
                        id = ((EnterpriseRepresentation.System)(node.Tag)).ID;

                        foreach (EnterpriseRepresentation.System sys in entrep.GetReferenceToAllSystems())
                        {

                            if (sys.ID.Equals(id))
                            {
                                sys.components.Clear();
                            }

                        }
                    }

                    this.PresentEnterpriseRepresentation();

                }
            }

        }

        private void EnterpriseRepresentationCreator_Load(object sender, EventArgs e)
        {
            ((MainMenu)(this.Parent)).Text = "Enteprise Creator";
        }

        private void entviewprocess_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                //entviewprocess.ContextMenu = new ContextMenu();
                ContextMenu menu = new ContextMenu();
                menu.MenuItems.Add("Set process criticality", new EventHandler(ChangeCriticality));

                Point ClickPoint = new Point(e.X, e.Y);
                TreeNode ClickNode = entviewprocess.GetNodeAt(ClickPoint);
                
                if (ClickNode == null) return;
                
                Point ScreenPoint = entviewprocess.PointToScreen(ClickPoint);
                Point FormPoint = this.PointToClient(ScreenPoint);
                
                if (ClickNode.Tag != null)
                {
                    if (ClickNode.Tag.GetType().ToString().Equals("EnterpriseRepresentation.BusinessProcess"))
                    {
                        currentbp = (BusinessProcess)ClickNode.Tag;
                        menu.Show(this, FormPoint);                                       
                    }
                }
            }
        }


        protected void ChangeCriticality(object sender, EventArgs e)
        {
            NumberInput numinp = new NumberInput(this);
            numinp.Show();
        }        
    }
}
