using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using EnterpriseRepresentation;

namespace EnterpriseAnalysis
{
    public class ProcessAnalyzer
    {
        public ProcessAnalyzer() {}

        EnterpriseRepresentation.EnterpriseRepresentation entrep;
        ComponentAnalyzer companalyzer = new ComponentAnalyzer();
        SystemAnalyzer sysanalysis = new SystemAnalyzer();
        EnvironmentAnalyzer envanalysis = new EnvironmentAnalyzer();

        public ArrayList GetCriticalComponentsByProcess(BusinessProcess bp)
        {
            ArrayList result = new ArrayList();
            bool process;

            foreach (ComponentAnalysisResult comp in result)
            {
                process = false;

                foreach (SubSystemOperation sysop in comp.comp.subsysops)
                {
                    if (companalyzer.GetProcessOf(sysop).Equals(bp) && companalyzer.IsCritical(sysop))
                    {
                        process = true;
                        break;
                    }
                }

                if (!process)
                {
                    result.Remove(comp);
                }
            }

            return result;
        } 

        public ArrayList GetProcessAnalysis(EnterpriseRepresentation.EnterpriseRepresentation entrep)
        {

            ArrayList result = new ArrayList();
            this.entrep = entrep;

            foreach (BusinessProcess bp in entrep.BusinessProcesses)
            {
                result.Add(new ProcessAnalysisResult(bp, GetAvailibilityOfProcess(bp), GetCriticalAvailibilityOfProcess(bp)));

            }

            return result;
        }

        public double GetAvailibilityOfProcess(BusinessProcess bp)
        {
            double availibility = 1.0;
            double compprob = 0.0;
            double sysprob = 0.0;
            double envprob = 0.0;

            ArrayList components = companalyzer.GetComponentAnalysis(entrep);
            ArrayList systems = sysanalysis.GetSystemAnalysis(entrep);
            ArrayList environments = envanalysis.GetEnvironmentAnalysis(entrep);

            foreach (SystemAnalysisResult sysres in systems)
            {
                    if (sysanalysis.SystemRealizesProcess(sysres.sys, bp))
                    {
                        sysprob = sysanalysis.GetAvgSystemProbability(sysres.sys, bp);
                        availibility *= ((sysprob * sysres.baseavailibility) + ((1.0 - sysprob)));                
                    }
            }

            

           foreach (ComponentAnalysisResult ccr in components)
           {
                if (companalyzer.ComponentRealizesProcess(ccr.comp, bp))
                {
                        
                    compprob = companalyzer.GetAvgComponentProbability(ccr.comp, bp);
                    availibility *= ((compprob * ccr.baseavailibility) + ((1.0 - compprob)));                      
                        
                }

           }

            foreach (EnvironmentAnalysisResult envres in environments)
           {
               if (envanalysis.EnvironmentRealizesProcess(envres.env, bp))
                {
                    envprob = envanalysis.GetAvgEnvironmentProbability(envres.env, bp);
                    availibility *= ((envprob * envres.baseavailibility) + ((1.0 - envprob)));
                }

           }




           return (availibility);
           

        }


        public double GetCriticalAvailibilityOfProcess(BusinessProcess bp)
        {
            double prob = 1.0;
            double compprob = 0.0;
            double sysprob = 0.0;
            double envprob = 0.0;

            ArrayList components = companalyzer.GetComponentAnalysis(entrep);
            ArrayList systems = sysanalysis.GetSystemAnalysis(entrep);
            ArrayList environments = envanalysis.GetEnvironmentAnalysis(entrep);

            foreach (SystemAnalysisResult sysres in systems)
            {
                    if (sysanalysis.SystemCriticallyRealizesProcess(sysres.sys, bp))
                    {
                        sysprob = sysanalysis.GetAvgSystemProbability(sysres.sys, bp);
                        prob *= ((sysprob * sysres.baseavailibility) + ((1.0 - sysprob)));
                    }
                
            }

                        
            foreach (ComponentAnalysisResult ccr in components)
            {
                if (companalyzer.ComponentCriticallyRealizesProcess(ccr.comp, bp))
                {

                    compprob = companalyzer.GetAvgComponentProbability(ccr.comp, bp);
                    prob *= ((compprob * ccr.baseavailibility) + ((1.0 - compprob)));

                }

            }

            
            foreach (EnvironmentAnalysisResult envres in environments)
            {
                if (envanalysis.EnvironmentCriticallyRealizesProcess(envres.env, bp))
                {
                    envprob = envanalysis.GetAvgEnvironmentProbability(envres.env, bp);
                    prob *= ((envprob * envres.baseavailibility) + ((1.0 - envprob)));

                }

            }




            return (prob);


        }







    }
}
