using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseRepresentation;
using System.Collections;

namespace EnterpriseAnalysis
{
    public class SystemAnalyzer
    {

        ArrayList result;
        EnterpriseRepresentation.EnterpriseRepresentation ent;

        public SystemAnalyzer() { }


        public ArrayList GetSystemAnalysis(EnterpriseRepresentation.EnterpriseRepresentation ent)
        {
            this.ent = ent;
            result = new ArrayList();
            ArrayList systems = ent.GetReferenceToAllSystems();

            foreach (EnterpriseRepresentation.System sys in systems)
            {
                double crit;
                double critsum = 0.0;
                double baseavb = GetAvailibilityOf(sys);

                EnterpriseRepresentation.Environment env;

                env = sys.env;
                double envavail = 1.0;
                
                while (env.prev != null)
                {
                    envavail *= GetAvailibilityOf(env);
                    env = env.prev;
                }

                double availibility = envavail * GetAvailibilityOf(sys);
                critsum = 0.0;

                foreach (BusinessProcess bp in ent.BusinessProcesses)
                {
                    crit = 0.0;
                    double prob = GetAvgSystemProbability(sys, bp);

                    foreach (SystemOperation sysop in sys.sysops)
                    {
                        if (IsCritical(sysop) && GetProcessOf(sysop).Equals(bp))
                        {

                            //double prob = GetProbabilityOf(sysop, bp);
                           
                            crit = bp.Criticality * prob;                          
                           
                         }
                    }

                    critsum += crit;
                }

            
                result.Add(new SystemAnalysisResult(sys, critsum, availibility, baseavb));                
                
                }
                        
            return result;
        }

        public bool SystemCriticallyRealizesProcess(EnterpriseRepresentation.System s, BusinessProcess bp)
        {           
            foreach (EnterpriseRepresentation.SystemOperation op in s.sysops)
            {
                if (GetProcessOf(op).Equals(bp) && this.IsCritical(op))
                {
                    return true;
                }
            }

            return false;
        }
        

        public bool SystemRealizesProcess(EnterpriseRepresentation.System s, BusinessProcess bp)
        {
            foreach (EnterpriseRepresentation.SystemOperation op in s.sysops)
            {
                if (GetProcessOf(op).Equals(bp))
                {
                    return true;
                }
            }

            return false;
        }

   

        public bool IsCritical(SystemOperation sysop)
        {

            bool critical = true;

            if (sysop.HasStereoType("NonCritical"))
            {
                critical = false;
            }

            if (sysop.businessfunction.HasStereoType("NonCritical"))
            {
                critical = false;
            }
       
            return critical;


        }


        public double GetAvgSystemProbability(EnterpriseRepresentation.System sys, BusinessProcess bp)
        {
           
           ArrayList bfs = new ArrayList();
      
           foreach (SystemOperation sysop in sys.sysops)
           {
            
             if (GetProcessOf(sysop).Equals(bp))
             {

                bool contains = false;

                foreach (BusinessFunction bf in bfs)
                {
                    if (bf.ID == sysop.businessfunction.ID)
                    {
                        contains = true;
                        break;
                    }
                }

                if (!contains)
                {
                    bfs.Add(sysop.businessfunction);
                }

              }
             }
            

            //Calculating how often the business processes are used in one execution of the Business Process.
            double sum = ent.GetProbabilityOfBusinessFunctions(bfs, bp.First, 1.0);

            //Returning the probability.
            return sum;

        }   



        public BusinessProcess GetProcessOf(SystemOperation sysop) { return sysop.businessfunction.process; }
        
        public double GetAvailibilityOf(EnterpriseRepresentation.EnterpriseEntity entity)
        {

            double avb = 1.0;

            if (entity.GotTagValue("MTBF") && entity.GotTagValue("MTTR"))
            {
                try
                {
                    double failurerate = double.Parse(entity.GetTagValue("MTBF"));
                    double repairtime = double.Parse(entity.GetTagValue("MTTR"));

                    avb = (failurerate / (failurerate + repairtime));
                }
                catch
                {                
                }
            }

            return avb;
        }



       


        

    }
}
