using System;
using System.Collections;
using System.Xml;
using System.Runtime.Serialization;

namespace EnterpriseRepresentation
{
    [Serializable]
	public class ComponentReader
	{
		XmlTextReader reader;
		XmlDocument document = new XmlDocument();
		Environment environment;

		public ComponentReader(){}

        public Component GetComponent(string filename, Environment env)
		{
			reader = new XmlTextReader(filename);
			document.Load(reader);
            environment = env;			
            return GetSuperComponent();
		}

        private Component GetSuperComponent()
		{
			Component thecomponent = null;
			XmlNodeList list = document.GetElementsByTagName("UML:Component");
			XmlNode node = list[0];
			
	        string name = node.Attributes["name"].Value;
					
			thecomponent = new Component(name,GetReferenceToSystem(environment,name),node.Attributes["xmi.id"].Value);

			if(node.HasChildNodes)
			{
						foreach(XmlNode subnode in node.ChildNodes)
						{
							if(subnode.Name.Equals("UML:Namespace.ownedElement"))
							{
								AppendSubComponents(subnode.ChildNodes,thecomponent);						
							}
						}
			}
				
			reader.Close();
			return thecomponent;
        }

		private string GetStereotypeNameByID(string id)
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:Stereotype"))
			{				
				if(node.Attributes["name"] != null)
				{
					if(node.Attributes["xmi.id"].Value.Equals(id))
					{
						return node.Attributes["name"].Value;
					}
				}
			}

			return "";
		}

		private void AppendSubComponents(XmlNodeList nodes,Component thecomponent)
		{
			foreach(XmlNode node in nodes)
			{				
				if(node.Name.Equals("UML:Component"))
				{
					
					Component c = new Component(node.Attributes["name"].Value,thecomponent,node.Attributes["xmi.id"].Value);
					thecomponent.components.Add(c);
					
					foreach(XmlNode subnode in node.ChildNodes)
					{
						if(subnode.Name.Equals("UML:Namespace.ownedElement"))
						{		
							AppendSubComponents(subnode.ChildNodes,c);	
						}
						else if(subnode.Name.Equals("UML:ModelElement.stereotype"))
						{
							foreach(XmlNode subsubnode in subnode)
							{
                                c.AddStereotype(GetStereotypeNameByID(subsubnode.Attributes["xmi.idref"].Value));								
							}
						}
						else if(subnode.Name.Equals("UML:ModelElement.taggedValue"))
						{
							foreach(XmlNode subsubnode in subnode)
							{
								if(subsubnode.Name.Equals("UML:TaggedValue"))
								{
									string tagname = subsubnode.Attributes["tag"].Value;
									string tagvalue = subsubnode.Attributes["value"].Value;						
									c.AddTag(new TagValue(tagname,tagvalue));	
								}
							}
						}
					}
				}
			}
		}

		private System GetReferenceToSystemByID(Environment env, string id)
		{
			foreach(Environment e in env.environments)
			{
				System temp = GetReferenceToSystemByID(e,id);

				if(temp != null)
					return temp;
			}

			foreach(System s in env.systems)
			{
				if(s.ID.Equals(id))
				{
					return s;
				}
			}

			return null;
		}

		private System GetReferenceToSystem(Environment env, string name)
		{
			foreach(Environment e in env.environments)
			{
				System temp = GetReferenceToSystem(e,name);

				if(temp != null)
					return temp;
			}

			foreach(System s in env.systems)
			{
				if(s.name.Equals(name))
				{
					return s;
				}
			}

			return null;
		}        
	}
}
