using System;
using System.Xml;
using System.Runtime.Serialization;

namespace EnterpriseRepresentation
{
    [Serializable]
	public class DeploymentReader
	{
		XmlTextReader reader;
		XmlDocument document = new XmlDocument();

		public DeploymentReader(){}

		public Environment GetDeployment(string filename)
		{
			reader = new XmlTextReader(filename);
			document.Load(reader);
			return GetSuperNode();
		}

		private Environment GetSuperNode()
		{
			Environment env = null;

			foreach(XmlNode node in document.GetElementsByTagName("UML:Node"))
			{
				if(node.Attributes["name"].Value.Equals("Enterprise"))
				{
					env = new Environment("Enterprise",null,node.Attributes["name"].Value);

					if(node.HasChildNodes)
					{
						foreach(XmlNode subnode in node.ChildNodes)
						{
							if(subnode.Name.Equals("UML:Namespace.ownedElement"))
							{
								env = GetSubNodes(subnode.ChildNodes,env);
							}
						}
					}
				}
			}

			reader.Close();
			return env;
		}

		private Environment GetSubNodes(XmlNodeList nodes, Environment env)
		{
			foreach(XmlNode node in nodes)
			{				
				if(node.Name.Equals("UML:Node"))
				{
					Environment e = new Environment(node.Attributes["name"].Value,env,node.Attributes["xmi.id"].Value);
					env.environments.Add(e);

					foreach(XmlNode subnode in node.ChildNodes)
					{
						if(subnode.Name.Equals("UML:Namespace.ownedElement"))
						{		
							GetSubNodes(subnode.ChildNodes,e);					
						}
						else if(subnode.Name.Equals("UML:Node.deployedComponent"))
						{
							foreach(XmlNode subsubnode in subnode.ChildNodes)
							{
								if(subsubnode.Name.Equals("UML:Component"))
								{
									string id = subsubnode.Attributes["xmi.idref"].Value;
									string name = GetComponentNameByID(id);
									  									
									System s = new System(name, e ,id);
                                    AppendStereotypesNameByComponentID(id, s);
									e.systems.Add(s);
									AddTags(s,id);
								}

							}

						}
						else if(subnode.Name.Equals("UML:ModelElement.taggedValue"))
						{
							foreach(XmlNode subsubnode in subnode.ChildNodes)
							{
								if(subsubnode.Name.Equals("UML:TaggedValue"))
								{
									string tag = subsubnode.Attributes["tag"].Value;
									string value = subsubnode.Attributes["value"].Value;

                                    e.AddTag(new TagValue(tag, value));							
								}
							}
						}	
					}	
				}						
			}
			return env;
		}

		private void AddTags(System system, string componentid)		
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:Component"))
			{

				string tagname = "";
				string tagvalue = "";

				if(node.Attributes["xmi.id"] != null)
				{
					if(node.Attributes["xmi.id"].Value.Equals(componentid))
					{
						foreach(XmlNode subnode in node)
						{
							if(subnode.Name.Equals("UML:ModelElement.taggedValue"))
							{
								foreach(XmlNode subsubnode in subnode)
								{
									if(subsubnode.Name.Equals("UML:TaggedValue"))
									{
										tagname = subsubnode.Attributes["tag"].Value;
										tagvalue = subsubnode.Attributes["value"].Value;

										system.AddTag(new TagValue(tagname,tagvalue));
									}
								}
							}
						}
					}
				}
			}
		}

        private void AppendStereotypesNameByComponentID(string id, System s)
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:Component"))
			{
				if(node.Attributes["xmi.id"] != null)
				{
					if(node.Attributes["xmi.id"].Value.Equals(id))
					{
						foreach(XmlNode subnode in node)
						{						
							if(subnode.Name.Equals("UML:ModelElement.stereotype"))
							{
                                foreach (XmlNode subsubnode in subnode)
                                {
                                    s.AddStereotype(GetStereotypeNameByID(subsubnode.Attributes["xmi.idref"].Value));
                                }
                            }

						}					
					}
				}
			}		
		}
        
		private string GetStereotypeNameByID(string id)
		{

			foreach(XmlNode node in document.GetElementsByTagName("UML:Stereotype"))
			{
				if(node.Attributes["xmi.id"] != null)
				{
					if(node.Attributes["xmi.id"].Value.Equals(id))
					{
						return node.Attributes["name"].Value;
					}
				}
			}

			return "";
		}

		private string GetComponentNameByID(string id)
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:Component"))
			{
				if(node.Attributes["xmi.id"] != null)
				{
					if(node.Attributes["xmi.id"].Value.Equals(id))
					{
						return node.Attributes["name"].Value;
					}
				}
			}
			return "";
		}
	}
}