using System;
using System.Collections;
using System.Runtime.Serialization;

namespace EnterpriseRepresentation
{
    [Serializable]
	public class EnterpriseRepresentation 
	{
        [Serializable]
        public enum ObjectType
        {
            ActivityState, ForkNode, MergeNode, DescsionNode, Final, NULL
        }

		public ArrayList processes = new ArrayList();
		public Environment environment;
		private string filename = "";
		public ComponentReader compreader;
		private DeploymentReader deployreader;
		private BusinessProcessReader processreader = new BusinessProcessReader();
		private SequenceReader seqreader;
        ArrayList allbusinessfunctions;

		public ArrayList BusinessProcesses
		{
			get{return processes;}
            set { processes = (ArrayList)value; }
		}

		public Environment Environment
		{
			get{return environment;}
            set { environment = (Environment)value; }
		}

		public EnterpriseRepresentation(){}

        public void CreateProcess(string name, int criticality)
        {
            processes.Add(new BusinessProcess(name,criticality));
        }

		public void AddProcess(string name, string filename, int crit)
		{
			this.filename=filename;
			BusinessProcess bp = processreader.GetProcess(filename,name);
            bp.criticality = crit;
            processes.Add(bp);
		}

		public void AddSystemOperations(string processname, string activityid,string filename)
		{		
			this.filename = filename;

			foreach(BusinessProcess bp in processes) 
	     	{
				if(bp.Name.Equals(processname))
				{                    
                    ParseSystemFunctions(bp.First, activityid, "");                   
				}
            }
		}

		public void AddSubSystemOperations(string process,string functionid,string operationid, string filename)
		{
			this.filename=filename;
						
			foreach(BusinessProcess p in processes)
			{				
				ParseSubSystemFunctions(p.First,functionid,operationid);			
			}
		}

		public void AddEnvironments(string filename)
		{
				deployreader = new DeploymentReader();
				environment = deployreader.GetDeployment(filename);
		}

		public void AddComponents(string filename)
		{
			compreader = new ComponentReader();

            Component comp = compreader.GetComponent(filename,Environment);

            foreach (Component c in comp.components)
            {
                GetReferanceToSystemByName(environment, comp.Name).AddComponent(c);
            }
        }


        private System GetReferanceToSystemByName(Environment env, string name)
        {
            foreach (Environment e in env.environments)
            {
                System temp = GetReferanceToSystemByName(e, name);

                if (temp != null)
                    return temp;
            }

            foreach (System s in env.systems)
            {
                if (s.Name.Equals(name))
                {
                    return s;
                }
            }

            return null;
        }

		private System GetReferanceToSystem(Environment env, string systemid)
		{
			foreach(Environment e in env.environments)
			{
				System temp = GetReferanceToSystem(e,systemid);

				if(temp != null)
					return temp;
			}

			foreach(System s in env.systems)
			{
				if(s.ID.Equals(systemid))
				{
					return s;
				}
			}
			return null;
		}


        public ArrayList GetReferenceToAllSystems()
        {
            ArrayList systems = new ArrayList();
            GetReferenceToAllSystems(this.Environment, systems);
            return systems;
        }

        public ArrayList GetReferenceToAllEnvironments()
        {
            ArrayList environ = new ArrayList();
            GetReferenceToAllEnvironments(this.Environment, environ);
            return environ;
        }

        public void RemoveSubComponentsFromComponent(string id)
        {
            foreach (Component comp in GetReferanceToAllComponents())
            {
                if (comp.ID.Equals(id))
                {
                    comp.components.Clear();
                }
            }
        }

        private void GetReferenceToAllEnvironments(Environment env, ArrayList environ)
        {
            environ.Add(env);

            foreach (Environment e in env.environments)
            {
                GetReferenceToAllEnvironments(e, environ);
            }           
        }

        private void GetReferenceToAllSystems(Environment env, ArrayList systems)
        {
            foreach (Environment e in env.environments)
            {
                GetReferenceToAllSystems(e, systems);
            }

            foreach (System system in env.systems)
            {
                systems.Add(system);
            }
        }

        public void RemoveSubSystemOperations(string bfid, string systemid, string name)
        {
            for (int i = 0; i < processes.Count; i++)
            {
                if (((BusinessProcess)(processes[i])).Name.Equals(name))
                {
                    foreach (BusinessFunction bf in this.GetEachUniqueBusinessFunction((BusinessProcess)(processes[i])))
                    {
                        if (bf.ID.Equals(bfid))
                        {
                            SystemOperation sysop = bf.FirstOperation;

                            while (sysop != null)
                            {
                                if (sysop.ID.Equals(systemid))
                                {
                                    sysop.firstoperation = null;
                                }

                                sysop = sysop.next;
                            }
                        }
                    }
                }
            }           
        }

		private void ParseSystemFunctions(object obj, string activityid, string operation)
		{			
			if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
			{
				BusinessFunction bf = ((BusinessFunction)(obj));
				
				if(bf.ID.Equals(activityid))
				{
					seqreader = new SequenceReader();
					bf.FirstOperation = seqreader.GetSystemOperations(environment,filename,bf);
				}
				
				if(bf.Next != null)
					ParseSystemFunctions(bf.Next,activityid,operation);				
			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionDecision"))
			{
				ArrayList nexts = (ArrayList)(((BusinessFunctionDecision)(obj)).Next);

				foreach(object o in nexts)
					ParseSystemFunctions(o,activityid,operation);
			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionFork"))
			{
				ArrayList nexts = (ArrayList)(((BusinessFunctionFork)(obj)).Next);

				foreach(object o in nexts)
					ParseSystemFunctions(o,activityid,operation);				

			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessMerge"))
			{
				BusinessFunctionMerge bm = (BusinessFunctionMerge)(obj);
				ParseSystemFunctions(bm.Next,activityid,operation);
			}
		}

        public void RemoveSystemFunction(string name, string fid)
        {
            for (int i = 0; i < processes.Count; i++)
            {
                if (((BusinessProcess)(processes[i])).Name.Equals(name))
                {
                    RemoveSystemFunction(((BusinessProcess)(processes[i])).First, fid);
                }
            }
        }

        private void RemoveSystemFunction(object obj, string activityid)
        {
            if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
            {
                BusinessFunction bf = ((BusinessFunction)(obj));

                if (bf.ID.Equals(activityid))
                {
                    bf.FirstOperation = null;
                }

                if (bf != null && bf.Next != null)
                    RemoveSystemFunction(bf.Next, activityid);
            }
            else if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionDecision"))
            {
                ArrayList nexts = (ArrayList)(((BusinessFunctionDecision)(obj)).Next);

                foreach (object o in nexts)
                    RemoveSystemFunction(o, activityid);
            }
            else if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionFork"))
            {
                ArrayList nexts = (ArrayList)(((BusinessFunctionFork)(obj)).Next);

                foreach (object o in nexts)
                    RemoveSystemFunction(o, activityid);

            }
            else if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessMerge"))
            {
                BusinessFunctionMerge bm = (BusinessFunctionMerge)(obj);
                RemoveSystemFunction(bm.Next, activityid);
            }          
        }
		
		private void ParseSubSystemFunctions(object obj, string functionid, string operationid)
		{			
			if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
			{
				BusinessFunction bf = ((BusinessFunction)(obj));
				
				if(bf.ID.Equals(functionid))
				{
					SystemOperation temp = bf.FirstOperation;

                    if (temp != null)
                    {

                        while (true)
                        {
                            if (temp.ID.Equals(operationid))
                            {
                                seqreader = new SequenceReader();
                                temp.firstoperation = seqreader.GetSubSystemOperations(temp.toosystem, temp.fromsystem, filename, temp);
                                break;
                            }
                            else
                            {
                                if (temp.next != null)
                                {
                                    temp = temp.next;
                                }
                                else
                                {
                                    break;
                                }
                            }
                        }

                    }
				}
				
				if(bf.Next != null)
					ParseSubSystemFunctions(bf.Next,functionid,operationid);				
			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionDecision"))
			{
				ArrayList nexts = (ArrayList)((BusinessFunctionDecision)(obj)).Next;

				foreach(object o in nexts)
					ParseSubSystemFunctions(o,functionid,operationid);
			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionFork"))
			{
				ArrayList nexts = (ArrayList)(((BusinessFunctionFork)(obj)).Next);

				foreach(object o in nexts)
					ParseSubSystemFunctions(o,functionid,operationid);				

			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessMerge"))
			{
				BusinessFunctionMerge bm = (BusinessFunctionMerge)(obj);
				ParseSubSystemFunctions(bm.Next,functionid,operationid);
			}
		}

        public ArrayList GetEachUniqueBusinessFunction(BusinessProcess bp)
        {
            ArrayList result = new ArrayList();
            GetEachUniqueBusinessFunction(bp.First,result);
            return result;
        }

        public bool ArrayListContainsByID(ArrayList list, BusinessFunction bf)
        {
            foreach(BusinessFunction obj in list)
            {
                if (obj.ID == bf.ID)
                    return true;
            }

            return false;
        }

        public void GetEachUniqueBusinessFunction(BusinessFunctionObject bfo, ArrayList result)
        {
            if (bfo.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
            {
                BusinessFunction bf = ((BusinessFunction)(bfo));

                if (!ArrayListContainsByID(result, bf))
                {
                    result.Add(bf);
                }

                if (bf.Next != null)
                    GetEachUniqueBusinessFunction(bf.Next,result);
            }
            else if (bfo.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionDecision"))
            {
                ArrayList nexts = (ArrayList)(((BusinessFunctionDecision)(bfo)).Next);

                foreach (BusinessFunctionObject o in nexts)
                    GetEachUniqueBusinessFunction(o, result);
            }
            else if (bfo.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionFork"))
            {
                ArrayList nexts = (ArrayList)(((BusinessFunctionFork)(bfo)).Next);

                foreach (BusinessFunctionObject o in nexts)
                    GetEachUniqueBusinessFunction(o, result);

            }
            else if (bfo.GetType().ToString().Equals("EnterpriseRepresentation.BusinessMerge"))
            {
                BusinessFunctionMerge bm = (BusinessFunctionMerge)(bfo);
                GetEachUniqueBusinessFunction(bm.Next, result);
            }
            
        }

        public static bool MultipleNexts(ObjectType obj)
        {
            if (obj == ObjectType.ForkNode || obj == ObjectType.DescsionNode)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public ObjectType GetObjectType(object obj)
        {
            if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
            {
                return ObjectType.ActivityState;
            }
            else if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionDecision"))
            {
                return ObjectType.DescsionNode;
            }
            else if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionFork"))
            {
                return ObjectType.ForkNode;
            }
            else if (obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionMerge"))
            {
                return ObjectType.MergeNode;
            }
            else
            {
                return ObjectType.NULL;
            }
        }


        public ArrayList GetReferanceToAllBusinnessFunctions(BusinessFunctionObject obj, BusinessFunction bf)
        {
            allbusinessfunctions = new ArrayList();
            GetReferanceToAllBusinnessFunctions(bf,obj);
            return allbusinessfunctions;
        }

        public ArrayList GetReferanceToAllComponents()
        {
            ArrayList result = new ArrayList();
            ArrayList systems = new ArrayList();

            GetReferenceToAllSystems(Environment, systems);
            
            foreach (System sys in systems)
            {
                result.AddRange(GetComponentsOf(sys.components));
            }

            return result;

        }

        public System GetSystemOfComponent(Component c)
        {
            if (c.parent.GetType().ToString().Equals("EnterpriseRepresentation.System"))
            {
                return (System)(c.parent);
            }
            else
            {
                return GetSystemOfComponent((Component)(c.parent));
            }
        }

        public ArrayList GetComponentsOf(ArrayList comps)
        {
            ArrayList result = new ArrayList();

            foreach(Component c in comps)
            {
                result.Add(c);
                result.AddRange(GetComponentsOf(c.components));
            }

            return result;         
        }

        private void GetReferanceToAllBusinnessFunctions(BusinessFunctionObject bf, BusinessFunctionObject obj)
        {
            ObjectType currentobject = GetObjectType(obj);

            if (currentobject == ObjectType.ActivityState)
            {
                if (((BusinessFunction)(obj)).ID.Equals(((BusinessFunction)(bf)).ID))
                {
                    allbusinessfunctions.Add(obj);
                }
            }

            if ((MultipleNexts(currentobject)))
            {
                ArrayList nexts = ((IFunctionMultipleNextObjects)(obj)).GetNexts();

                foreach (BusinessFunctionObject bfo in nexts)
                {
                    GetReferanceToAllBusinnessFunctions(bf, bfo);
                }
            }
            else
            {
                if (((IFunctionNextAccessor)(obj)).Next != null)
                {
                    GetReferanceToAllBusinnessFunctions(bf,((IFunctionNextAccessor)(obj)).Next);
                }
            }
        }




        public double GetProbabilityOfBusinessFunctions(ArrayList bfs, BusinessFunctionObject obj, double sum)
        {
            ObjectType currentobject = GetObjectType(obj);
            double partialsum = 0.0;

            if (currentobject == ObjectType.ActivityState)
            {

                foreach (BusinessFunction bf in bfs)
                {
                    if (bf.ID == ((BusinessFunction)(obj)).ID)
                    {
                        return sum;
                    }
                }
            }
          
            if ((MultipleNexts(currentobject)))
            {
                ArrayList nexts = ((IFunctionMultipleNextObjects)(obj)).GetNexts();
                ArrayList probs = null;

                if (currentobject == ObjectType.DescsionNode)
                {
                    probs = ((BusinessFunctionDecision)(obj)).GetProbs();

                    for (int i = 0; i < nexts.Count; i++)
                    {
                        partialsum += GetProbabilityOfBusinessFunctions(bfs, (BusinessFunctionObject)nexts[i], sum * (double)probs[i]);    
                    }

                    return partialsum;

                }
                else
                {
                    for (int i = 0; i < nexts.Count; i++)
                    {
                        partialsum += GetProbabilityOfBusinessFunctions(bfs, (BusinessFunctionObject)nexts[i], sum);
                    }

                    return partialsum;
                }

                
            }
            else
            {
                if (((IFunctionNextAccessor)(obj)).Next != null)
                {
                    //No branching. The sum is passed downwards. No contribution.
                    return GetProbabilityOfBusinessFunctions(bfs, ((IFunctionNextAccessor)(obj)).Next, sum);
                }
            }

            //No contribution. The branch is not realized by a Business Function in bfs
            return 0.0;
        }


	}
}
