using System;
using System.Collections;
using System.Xml;

namespace RiskAnalysis
{
	/// <summary>
	/// Summary description for ActivityDiagram.
	/// </summary>
	public class ActivityDiagram : Diagram
	{
		public ArrayList actionstates;
		public ArrayList transitions;
		public ArrayList stereotypes;
        XmlDocument document;

		private System.Xml.XmlTextReader reader;
		string filename = "";

		public ActivityDiagram(string filename)
		{
			this.filename = filename;
            reader = new XmlTextReader(filename);
            document = new XmlDocument();
            document.Load(reader);
			actionstates = new ArrayList();
			transitions = new ArrayList();
            ParseStereotypes(filename);
            GetActionStates();
			GetTransitions();		
			
		}

		public void GetTransitions()
		{

			//string guard = "";
			string source = "";
			string target = "";
			string id = "";
			string name = "";

			reader = new System.Xml.XmlTextReader(filename);


            XmlNodeList nl = document.GetElementsByTagName("UML:Transition");

			foreach(XmlNode node in nl)
            {
                source = node.Attributes["source"].Value;
                target = node.Attributes["target"].Value;
                id = node.Attributes["xmi.id"].Value;
                name = node.Attributes["name"].Value;

                ActivityTransition at = new ActivityTransition(GetActivityStateByID(target), GetActivityStateByID(source), id, name);
                ExtractStereotypeAndTags(node,at);
                transitions.Add(at);

            }
		}

		/*public void GetStereotypes()
		{

			XmlTextReader reader = new System.Xml.XmlTextReader(filename);
			string id = "";
			string name = "";

            reader = new System.Xml.XmlTextReader(filename);
            XmlDocument document = new XmlDocument();
            document.Load(reader);
            XmlNodeList nl = document.GetElementsByTagName("UML:Stereotype");

            foreach (XmlNode node in nl)
            {
                if (node.Attributes["xmi.id"] != null)
                {
                    name = node.Attributes["name"].Value;
                    id = node.Attributes["xmi.id"].Value;
                    stereotypes.Add(new Stereotype(id, name));
                }
		    }
		}*/


        public ActivityState GetActivityStateByID(string id)
        {
            foreach (ActivityState state in actionstates)
            {
                //if (state.stereotypes != null)
                //{
                    if (state.ID.Equals(id))
                    {
                        return state;
                    }
                //}
            }

            return null;


        }


		public ArrayList GetActivityStatesByStereotype(string name)
		{
			ArrayList output = new ArrayList();

			foreach(ActivityState state in actionstates)
			{
                foreach (Stereotype s in state.stereotypes)
                {
                    if (s.name.Equals(name))
                    {
                        output.Add(state);
                        break;
                    }
                }
			}

			return output;
		}


		public void GetActionStates()
		{
			string name = "";
			string id = "";
			reader = new System.Xml.XmlTextReader(filename);
			XmlDocument document = new XmlDocument();
			document.Load(reader);

			XmlNodeList nl = document.GetElementsByTagName("UML:ActionState");
			
			foreach(XmlNode node in nl)
			{
				name = node.Attributes["name"].Value;				
				id = node.Attributes["xmi.id"].Value;
                ActivityState actionstate = new ActivityState(name,id,new ArrayList());

				if(node.HasChildNodes)
				{
					ExtractStereotypeAndTags(node,actionstate);					
				}

				actionstates.Add(actionstate);
			}
		}
	}
}


