using System;
using System.Data.OleDb;
using System.Data;

namespace RiskAnalysis
{
	/// <summary>
	/// Summary description for HazopConfigurationHandler.
	/// </summary>
	public class HazopTransformationProfileManager
	{
		
		//Class members
		OleDbConnection conn = new OleDbConnection();
		OleDbDataAdapter adapter = new OleDbDataAdapter();
		OleDbCommand command = new OleDbCommand();
		string StartUpPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);
		String sql = "";
		
		//.....
		public HazopTransformationProfileManager()
		{
			SetUpDatabaseConnection();
		}

		public void SetUpDatabaseConnection()
		{
			//Sets the connection string
			conn.ConnectionString=@"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + StartUpPath + "\\hazopconfig.mdb";
		}


		//Methods for adding data
		public void AddGuideWord(string name, int parameter)
		{
			conn.Open();
			sql = "INSERT INTO Guideword(Name, Parameter) VALUES('" + name + "',"  + parameter + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

		public void AddParameter(string name,int studynode)
		{
			conn.Open();
			sql = "INSERT INTO Parameter(Name, Studynode) VALUES('" + name + "',"  + studynode + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();

		}
		
		public void AddStudyNode(string name,int profile, int extends)
		{

			conn.Open();
			sql = "INSERT INTO StudyNode(Name, Profile, ExtendsEntity) VALUES('" + name + "',"  + profile + "," + extends + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();

		}
		
		public void AddProfile(string name, int diagramtype)
		{
			conn.Open();
			sql = "INSERT INTO HazopProfile(Name, DiagramType) VALUES('" + name + "',"  + diagramtype + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

		
		//Methods for removing data.
		public void RemoveGuideWord(int id)
		{
			RemoveSingleRow("Guideword",id);
		}

		public void RemoveParameter(int id)
		{
			RemoveSingleRow("Parameter",id);
			RemoveGuidewordsByParameter(id);
		}

		public void RemoveStudyNode(int id)
		{

			//ALT FEIL!!!!!

			RemoveSingleRow("StudyNode",id);
			
			//COMMENT
				
			conn.Open();
			sql = "DELETE FROM Guideword WHERE Guideword.Parameter IN(SELECT ID FROM Parameter WHERE studynode = " +id+")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();

			//COMMENT
			RemoveParametersByStudyNode(id);

		}

		public void RemoveHazopProfile(int id)
		{
			//Deletes the Profile
			RemoveSingleRow("HazopProfile",id);
			
			conn.Open();
			
			//Deletes all the guidewords connected with the Profile
			sql = "DELETE FROM GuideWord WHERE GuideWord.Parameter IN ";
			sql += "(SELECT ID FROM Parameter WHERE Studynode IN ";
			sql += "(SELECT ID FROM StudyNode WHERE Profile =" + id + "))";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			
			//Deletes all the parameters connected with the profile
			sql = "DELETE FROM Parameter WHERE Parameter.Studynode IN ";
			sql += "(SELECT ID FROM StudyNode WHERE Profile = " + id + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		
			//Deletes all the study nodes connected with a profile.
			RemoveStudyNodeByProfile(id);
		}

		public void RemoveParametersByStudyNode(int id)
		{
			RemoveMultipleRows("Parameter","Studynode",id);
		}

		public void RemoveStudyNodeByProfile(int id)
		{
			RemoveMultipleRows("Studynode","Profile",id);
		}

		public void RemoveGuidewordsByParameter(int id)
		{
			RemoveMultipleRows("Guideword","Parameter",id);
		}



		//Methods for getting data
		public DataTable GetStudyNodes(int profile)
		{
			
			return GetDataTableFromDatabase("SELECT StudyNode.ID,StudyNode.Name,UMLEntityType.TypeName FROM StudyNode, UMLEntityType WHERE UMLEntityType.ID = StudyNode.ExtendsEntity AND StudyNode.Profile = " + profile);
			
			
		}

		public DataTable GetParameters(int studynode)
		{
			return GetDataTableFromDatabase("SELECT ID, Name FROM Parameter WHERE Studynode =" + studynode);
		}

		public DataTable GetParametersByName(string studynode)
		{
			return GetDataTableFromDatabase("SELECT Parameter.ID, Parameter.Name, StudyNode.Name FROM Parameter, StudyNode WHERE Parameter.StudyNode = StudyNode.ID AND StudyNode.Name ='" + studynode + "'");
		}

		public DataTable GetDiagramTypes()
		{
			return GetDataTableFromDatabase("SELECT ID,NAME FROM DiagramType");
		}

		public DataTable GetProfiles()
		{			
			return GetDataTableFromDatabase("SELECT HazopProfile.ID,HazopProfile.Name,DiagramType.Name,DiagramType.ID FROM HazopProfile LEFT JOIN DiagramType ON DiagramType.ID = HazopProfile.DiagramType");			
		}

        public DataTable GetProfilesByID(string id)
        {
            return GetDataTableFromDatabase("SELECT ID, Name, DiagramType FROM HazopProfile WHERE DiagramType =" + id);
        }

		public DataTable GetUMLEntityTypes(int diagramtype)
		{
			return GetDataTableFromDatabase("SELECT ID,TypeName FROM UMLEntityType WHERE DiagramType =" + diagramtype);
		}

		public DataTable GetGuideWords(int parameter)
		{
			return GetDataTableFromDatabase("SELECT ID,NAME FROM Guideword WHERE Parameter =" + parameter);
		}	

		//Database interface methods.
		private DataTable GetDataTableFromDatabase(string sql)
		{
			conn.Open();						
			DataSet data = new DataSet();			
			adapter = new OleDbDataAdapter(sql,conn);
			adapter.Fill(data);
			conn.Close();			
			return data.Tables[0];
		}
		private void RemoveSingleRow(string table,int id)
		{
			conn.Open();
			sql = "DELETE FROM " + table + " WHERE ID=" + id;
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

		private void RemoveMultipleRows(string table, string attributt, int id)
		{
			conn.Open();
			sql = "DELETE FROM " + table + " WHERE " + attributt + "=" + id;
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

	}
}
