using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Xml;

namespace RiskAnalysis
{
    public partial class PHAOutputPresenter : UserControl
    {
        ArrayList outputs = new ArrayList();
        PHATransformationProfileManager config = new PHATransformationProfileManager();
        PHATransformer phatransformer = new PHATransformer();

        DataTable dtdiagramtype;
        DataTable dtprofiles;

        public PHAOutputPresenter()
        {
            InitializeComponent();

            FillDiagramTypes();

            ColumnHeader ch;

            ch = new ColumnHeader();
            ch.Width = 100;
            ch.Text = "StudyNode";            
            listview.Columns.Add(ch);

            ch = new ColumnHeader();
            ch.Width = 100;
            ch.Text = "Name";
            listview.Columns.Add(ch);

            ch = new ColumnHeader();
            ch.Width = 100;
            ch.Text = "Descrption";
            listview.Columns.Add(ch);

            listview.View = View.Details;
        }


        public void FillProfiles(string profile)
        {
            dtprofiles = config.GetProfilesByID(profile);

            if (dtprofiles.Rows.Count > 0)
            {
                for (int i = 0; i < dtprofiles.Rows.Count; i++)
                {
                    profiles.Items.Add(dtprofiles.Rows[i][1]);
                }

                profiles.SelectedIndex = 0;
            }

        }

        public void FillDiagramTypes()
        {

            dtdiagramtype = config.GetDiagramTypes();

            if (dtdiagramtype != null)
            {
                for (int i = 0; i < dtdiagramtype.Rows.Count; i++)
                {
                     diagramtypes.Items.Add(dtdiagramtype.Rows[i][1]);
                }
            }
            
            diagramtypes.SelectedIndex = 0;
            FillProfiles((string)(dtdiagramtype.Rows[0][0].ToString()));		

        }


        public void FillListView()
        {
            listview.Items.Clear();

            foreach (PHAOutputItem poi in outputs)
            {
                ListViewItem item = new ListViewItem(new string[] {poi.studynode,poi.checklistname,poi.checklistdesc});
                listview.Items.Add(item);

            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (listview.SelectedIndices.Count>0)
            {
                
                outputs.RemoveAt(listview.SelectedIndices[0]);
                listview.Items.RemoveAt(listview.SelectedIndices[0]);        
                FillListView();
            }
        }       

        private void diagramtypes_SelectedIndexChanged(object sender, EventArgs e)
        {
           profiles.Items.Clear();
           FillProfiles((string)(dtdiagramtype.Rows[diagramtypes.SelectedIndex][0].ToString()));
        }

        private void button2_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
        }

        private void analyze_Click(object sender, EventArgs e)
        {
            try
            {
                XmlTextReader reader = new System.Xml.XmlTextReader(openFileDialog1.FileName);
                XmlDocument document = new XmlDocument();
                document.Load(reader);
                XmlNodeList nl = document.GetElementsByTagName("UML:Diagram");

                if (nl.Count > 1)
                {
                    MessageBox.Show("The XMI file contains more than one diagram");
                }
                else if (nl.Count == 0)
                {
                    MessageBox.Show("The XMI file does not contain any diagrams");
                }
                else if (openFileDialog1.FileName.Length > 0)
                {
                    if (diagramtypes.SelectedItem.ToString().Equals("Component Diagram"))
                    {
                        outputs = phatransformer.ParseComponentDiagram((int)(dtprofiles.Rows[profiles.SelectedIndex][0]), openFileDialog1.FileName);
                    }
                    else if (diagramtypes.SelectedItem.ToString().Equals("Activity Diagram"))
                    {
                        outputs = phatransformer.ParseActivityDiagram((int)(dtprofiles.Rows[profiles.SelectedIndex][0]), openFileDialog1.FileName);
                    }
                    else if (diagramtypes.SelectedItem.ToString().Equals("Deployment Diagram"))
                    {
                        outputs = phatransformer.ParseDeploymentDiagram((int)(dtprofiles.Rows[profiles.SelectedIndex][0]), openFileDialog1.FileName);
                    }

                    FillListView();
                }
            }
            catch {}
           
        }

        private void export_Click(object sender, EventArgs e)
        {
            
            RiskAnalysis.ReportMaker rm = new ReportMaker();
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                rm.AppendLine("PHAOutput");
                rm.AppendBreak();
                DataTable datatab = new DataTable();

                datatab.Columns.Add("Studynode");
                datatab.Columns.Add("Name");
                datatab.Columns.Add("Description");

                foreach (PHAOutputItem poi in outputs)
                {
                    datatab.Rows.Add(new string[] { poi.studynode, poi.checklistname, poi.checklistdesc });
                }

                rm.AppendDataTable(datatab);
                rm.SaveReport(saveFileDialog1.FileName);
            }           
            
        }

        private void addrisk_Click(object sender, EventArgs e)
        {
            AddRiskDialogue ard = new AddRiskDialogue();

            if (listview.SelectedIndices.Count > 0)
            {
                int index = listview.SelectedIndices[0];
                PHAOutputItem item = (PHAOutputItem)outputs[index];
                ard.SetName(item.checklistname);
                ard.SetDescription(item.studynode + " " + item.checklistdesc);
                ard.Show();
            }
        }

        private void back_Click(object sender, EventArgs e)
        {
            ((MainForm)(this.Parent)).SetUserControl("Menu");
        }

     
            
    }
}
