using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseRepresentation;
using System.Collections;

namespace EnterpriseAnalysis
{
    public class EnvironmentAnalyzer
    {

        public EnvironmentAnalyzer() { }

        EnterpriseRepresentation.EnterpriseRepresentation ent;
        ArrayList allbusinessfunctions = new ArrayList();
        SystemAnalyzer sysanalzer = new SystemAnalyzer();
           
        public ArrayList GetEnvironmentAnalysis(EnterpriseRepresentation.EnterpriseRepresentation ent)
        {
            this.ent = ent;
            ArrayList result = new ArrayList();
            ArrayList environments = ent.GetReferenceToAllEnvironments();
            double crit;
            

            foreach (EnterpriseRepresentation.Environment e in environments)
            {
                crit = 0.0;
                double baseavb = GetAvailibilityOf(e);               
                
                foreach (BusinessProcess bp in ent.BusinessProcesses)
                {                                       
                    if(EnvironmentRealizesProcess(e,bp))
                    {
                        double prob = GetAvgEnvironmentProbability(e, bp);
                        crit += bp.Criticality * prob;
                    }
                }

                double availibility = GetEnvironmentAvailibility(e);

                EnterpriseRepresentation.Environment temp = e;

                while (temp.prev != null)
                {
                    availibility *= GetEnvironmentAvailibility(temp.prev);
                    temp = temp.prev;
                }
                

                result.Add(new EnvironmentAnalysisResult(e, crit, GetEnvironmentAvailibility(e),baseavb));
            }
            

            return result;
        }


        public double GetAvailibilityOf(EnterpriseRepresentation.EnterpriseEntity entity)
        {
            double avb = 1.0;

            if (entity.GotTagValue("MTBF") && entity.GotTagValue("MTTR"))
            {
                try
                {
                    double failurerate = double.Parse(entity.GetTagValue("MTBF"));
                    double repairtime = double.Parse(entity.GetTagValue("MTTR"));

                    avb = (failurerate / (failurerate + repairtime));
                }
                catch{}
            }

            return avb;
        }


        public double GetEnvironmentAvailibility(EnterpriseRepresentation.Environment e)
        {

            double availibility = 1.0;

            availibility *= GetAvailibilityOf(e);
            
            while (e.prev != null)
            {
                e = e.prev;
                availibility *= GetAvailibilityOf(e);
            }

            return availibility;

        }

        public bool IsCritical(SystemOperation sysop)
        {

            bool critical = true;

            if (sysop.HasStereoType("NonCritical"))
            {
                critical = false;
            }

            if (sysop.businessfunction.HasStereoType("NonCritical"))
            {
                critical = false;
            }


            return critical;


        }



        public bool EnvironmentCriticallyRealizesProcess(EnterpriseRepresentation.Environment e, BusinessProcess bp)
        {
            ArrayList systems = new ArrayList();
            GetSystemsOfEnvironment(e, systems);

            foreach (EnterpriseRepresentation.System s in systems)
            {
                foreach (EnterpriseRepresentation.SystemOperation op in s.sysops)
                {
                    BusinessProcess test = GetProcessOf(op);

                    if (GetProcessOf(op).Equals(bp) && IsCritical(op))
                    {
                        return true;
                    }

                }
            }
                      
            return false;
        }

        public bool EnvironmentRealizesProcess(EnterpriseRepresentation.Environment e, BusinessProcess bp)
        {

            ArrayList systems = new ArrayList();
            GetSystemsOfEnvironment(e,systems);

            foreach (EnterpriseRepresentation.System s in systems)
            {
                foreach (EnterpriseRepresentation.SystemOperation op in s.sysops)
                {
                    BusinessProcess test = GetProcessOf(op);

                    if (GetProcessOf(op).Equals(bp))
                    {
                        return true;
                    }

                }
            }
                      
            return false;
        }


        public void GetSystemsOfEnvironment(EnterpriseRepresentation.Environment e, ArrayList systems)
        {
            foreach (EnterpriseRepresentation.System sys in e.systems)
            {
                systems.Add(sys);
            }

            foreach(EnterpriseRepresentation.Environment env in e.environments)
            {
                GetSystemsOfEnvironment(env, systems);
            }
        }

        public double GetAvgEnvironmentProbability(EnterpriseRepresentation.Environment e, BusinessProcess bp)
        {
            //A list for storing system referances.
            ArrayList systems = new ArrayList();

            //A list for storing business functions
            ArrayList bfs = new ArrayList();
            
            //Getting a list of all the systems running under Environment e.
            GetSystemsOfEnvironment(e, systems);
            
            //Extracting the business processes used by the different systems (and therefore the given Environment)
            foreach (EnterpriseRepresentation.System s in systems)
            {
                foreach (SystemOperation sysop in s.sysops)
                {
                    if (GetProcessOf(sysop).Equals(bp))
                    {

                        bool contains = false;

                        foreach (BusinessFunction bf in bfs)
                        {
                            if (bf.ID == sysop.businessfunction.ID)
                            {
                                contains = true;
                                break;
                            }
                        }

                        if (!contains)
                        {
                            bfs.Add(sysop.businessfunction);
                        }
                    }
                }
            }

            //Calculating how often the business processes are used in one execution of the Business Process.
            double sum = ent.GetProbabilityOfBusinessFunctions(bfs, bp.First, 1.0);
            
            //Returning the probability.
            return sum;

        }
             

        public EnterpriseRepresentation.BusinessProcess GetProcessOf(SystemOperation sysop)
        {
            return sysop.businessfunction.process;
        }
    }
}
