using System;
using System.Xml;
using System.Collections;
using System.Runtime.Serialization;

namespace EnterpriseRepresentation
{
    [Serializable]
	public class BusinessProcessReader
	{
		XmlTextReader reader;
		XmlDocument document = new XmlDocument();
		BusinessProcess process;

        [Serializable]
		
        public enum ObjectType
		{
			ActivityState, ForkNode, MergeNode, DescsionNode, Final, NULL
		}

		public BusinessProcessReader(){}

		public BusinessProcess GetProcess(string filename, string name)
		{
			reader = new XmlTextReader(filename);
			process = new BusinessProcess();
            process.Name = name;
			ParseProcess(process);
			reader.Close();
			return process;
		}

		private void ParseProcess(BusinessProcess process)
		{
			document.Load(reader);
			string initialid = GetInitialNodeID();
			string current = GetNextID(initialid);            						
			ObjectType firstobject = GetObjectType(current);            
            string next = GetNextID(current);
           

            if (firstobject == ObjectType.ActivityState)
            {
                process.First = (BusinessFunctionObject)(new BusinessFunction(GetActivityStateName(current), process, current));
                GetActivityStateStereotypes(current, (BusinessFunction)(process.First));
                GetActivityTags(current, ((BusinessFunction)(process.First)).Tags);

                BusinessFunctionObject nextobj;

                if (GetObjectType(next) == ObjectType.DescsionNode)
                {
                    nextobj = new BusinessFunctionDecision();
                 }
                else if (GetObjectType(next) == ObjectType.ForkNode)
                {
                    nextobj = new BusinessFunctionFork();
                }
                else
                {
                    nextobj = (BusinessFunctionObject)(new BusinessFunction(GetActivityStateName(next), process, next));
                    GetActivityStateStereotypes(current, (BusinessFunction)(nextobj));
                    GetActivityTags(current, ((BusinessFunction)(nextobj)).Tags);
                }

                ((BusinessFunction)(process.First)).Next = nextobj;
                ParseFunctionObject(nextobj,process.First, next);
            }
            else if (MultipleNexts(firstobject))
            {

                if (firstobject == ObjectType.DescsionNode)
                {
                    process.First = new BusinessFunctionDecision();
                }
                else
                {
                    process.First = new BusinessFunctionFork();
                }

                foreach (string id in GetTransitionTargetsFrom(current))
                {

                    ObjectType nextobject = GetObjectType(id);
                    BusinessFunctionObject nextobj = null;

                    if (nextobject == ObjectType.ActivityState)
                    {
                        nextobj = new BusinessFunction(GetActivityStateName(id), process, id);
                        GetActivityStateStereotypes(id, (BusinessFunction)(nextobj));
                        GetActivityTags(id, ((BusinessFunction)(nextobj)).Tags);
                       
                    }
                    else if (nextobject == ObjectType.MergeNode)
                    {
                        nextobj = new BusinessFunctionMerge();
                    }
                    else if (nextobject == ObjectType.DescsionNode)
                    {
                        nextobj = new BusinessFunctionDecision();

                    }
                    else if (nextobject == ObjectType.ForkNode)
                    {
                        nextobj = new BusinessFunctionFork();
                    }

                    if (nextobj != null)
                    {
                        if (firstobject == ObjectType.DescsionNode)
                        {
                            ((BusinessFunctionDecision)(process.First)).AddProb(GetProbabilityOf(current, id));
                        }

                        ((IFunctionMultipleNextObjects)(process.First)).AddNext(nextobj);
                        ParseFunctionObject(nextobj, process.First, id);
                    }
                }
            }
            
		}
   
        private void ParseFunctionObject(object function, object prev, string current)
        {
            //Gets the object type of the current object
            ObjectType curobject = GetObjectType(function);
            
            //Sets the current.prev = prev
            ((BusinessFunctionObject)function).Prev = (BusinessFunctionObject)(prev);
           
            if (MultipleNexts(curobject))
            {
                foreach (string id in GetTransitionTargetsFrom(current))
                {
                    ObjectType nextobject = GetObjectType(id);
                    BusinessFunctionObject next = null;

                    if (nextobject == ObjectType.ActivityState)
                    {
                        next = new BusinessFunction(GetActivityStateName(id), process, id);
                        GetActivityStateStereotypes(id, (BusinessFunction)(next));
                        GetActivityTags(id, ((BusinessFunction)(next)).Tags);
                    }
                    else if (nextobject == ObjectType.MergeNode)
                    {
                        next = new BusinessFunctionMerge();
                    }
                    else if (nextobject == ObjectType.DescsionNode)
                    {
                        next = new BusinessFunctionDecision();

                    }
                    else if (nextobject == ObjectType.ForkNode)
                    {
                        next = new BusinessFunctionFork();
                    }

                    if (next != null)
                    {

                        if (curobject == ObjectType.DescsionNode)
                        {
                            ((BusinessFunctionDecision)(function)).AddProb(GetProbabilityOf(current, id));
                        }

                        ((IFunctionMultipleNextObjects)(function)).AddNext(next);
                        ParseFunctionObject(next, function, id);
                    }
                }
            }
            else
            {
                string nextid = GetNextID(current);
                ObjectType nexttype = GetObjectType(nextid);
                BusinessFunctionObject next = null;


                if (MultipleNexts(nexttype))
                {

                    if (nexttype == ObjectType.DescsionNode)
                    {
                        next = new BusinessFunctionDecision();
                    }
                    else
                    {
                        next = new BusinessFunctionFork();
                    }
                }
                else
                {
                    if (nexttype == ObjectType.ActivityState)
                    {
                        next = new BusinessFunction(GetActivityStateName(nextid), process, nextid);
                        GetActivityStateStereotypes(nextid, (BusinessFunction)(next));
                        GetActivityTags(nextid, ((BusinessFunction)(next)).Tags);
                    }
                    else if (nexttype == ObjectType.MergeNode)
                    {
                        next = new BusinessFunctionMerge();
                    }
                }

                if (next != null)
                {
                    ((IFunctionNextAccessor)(function)).Next = next;
                    ParseFunctionObject(((IFunctionNextAccessor)(function)).Next, function, nextid);
                }
            }
        }

		private ArrayList GetTransitionTargetsFrom(string source)
		{
			ArrayList targets = new ArrayList();

			foreach(XmlNode node in document.GetElementsByTagName("UML:Transition"))
			{
				if(node.Attributes["source"].Value.Equals(source))
				{
					targets.Add(node.Attributes["target"].Value);
				}			
			}	
	
			return targets;

		}

		private ObjectType GetObjectType(string id)
		{
		
			foreach(XmlNode node in document.GetElementsByTagName("UML:ActionState"))
			{
				if(node.Attributes["xmi.id"].Value.Equals(id))
				{
					return ObjectType.ActivityState;
				}			
			}

			foreach(XmlNode node in document.GetElementsByTagName("UML:Pseudostate"))
			{
				if(node.Attributes["xmi.id"].Value.Equals(id))
				{
					if(node.Attributes["kind"].Value.Equals("junction"))
					{
						if(IsADecisionNode(id))
						{
							return ObjectType.DescsionNode;				
						}
						else
						{
							return ObjectType.MergeNode;
						}
					}	
					
					else if(node.Attributes["kind"].Value.Equals("fork"))
					{
						return ObjectType.ForkNode;			
					}	

				}			
			}

			foreach(XmlNode node in document.GetElementsByTagName("UML:Pseudostate"))
			{
				if(node.Attributes["xmi.id"].Value.Equals(id))
				{
					if(node.Attributes["kind"].Value.Equals("final"))
					{
						return ObjectType.Final;				
					}	
				}
			}

			return ObjectType.NULL;
		}

		private string GetInitialNodeID()
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:Pseudostate"))
			{
				if(node.Attributes["kind"].Value.Equals("initial"))
				{
					return node.Attributes["xmi.id"].Value;
				}
			}

			return null;
		}

		private bool IsADecisionNode(string id)
		{
			int numofinputs = 0;

			foreach(XmlNode node in document.GetElementsByTagName("UML:Transition"))
			{
				if(node.Attributes["target"].Value.Equals(id))
				{
					numofinputs++;
				}
			}
			
			return(numofinputs == 1);

		}

		private string GetNextID(string id)
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:Transition"))
			{
				if(node.Attributes["source"].Value.Equals(id))
				{
					return node.Attributes["target"].Value;
				}
			}

			return null;	
		}

		private string GetActivityStateName(string id)
		{

			foreach(XmlNode node in document.GetElementsByTagName("UML:ActionState"))
			{
				if(node.Attributes["xmi.id"].Value.Equals(id))
				{
					return node.Attributes["name"].Value;
				}
			}

			return "";
		}

		public ObjectType GetObjectType(object obj)
		{
			if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunction"))
			{
				return ObjectType.ActivityState;			
			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionDecision"))
			{
				return ObjectType.DescsionNode;
			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionFork"))
			{
				return ObjectType.ForkNode;
			}
			else if(obj.GetType().ToString().Equals("EnterpriseRepresentation.BusinessFunctionMerge"))
			{
				return ObjectType.MergeNode;
			}
			else
			{
				return ObjectType.NULL;
			}
		}

        public double GetProbabilityOf(string fromobj, string toobj)
        {
            foreach (XmlNode node in document.GetElementsByTagName("UML:Transition"))
            {
                if (node.Attributes["target"].Value.Equals(toobj) && node.Attributes["source"].Value.Equals(fromobj))
                {
                    try
                    {
                        return double.Parse(node.Attributes["name"].Value);                 
                    }
                    catch {

                        try
                        {
                            return double.Parse(node.Attributes["name"].Value.Replace('.', ','));
                        }
                        catch { return 0.0;}
                    }
                }
            }

            return 0.0;
        }

		public static bool MultipleNexts(ObjectType obj)
		{
			if(obj == ObjectType.ForkNode || obj == ObjectType.DescsionNode)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		private void GetActivityTags(string id, ArrayList tags)
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:ActionState"))
			{
				if(node.Attributes["xmi.id"].Value.Equals(id))
				{
					foreach(XmlNode subnode in node)
					{
						if(subnode.Name.Equals("UML:ModelElement.taggedValue"))
						{
							foreach(XmlNode subsubnode in subnode)
							{								
								tags.Add(new TagValue(subsubnode.Attributes["tag"].Value,subsubnode.Attributes["value"].Value));
							}
							break;
						}	
					}
					break;
				}
			}
		}

        private void GetActivityStateStereotypes(string id, BusinessFunction bf)
		{

			XmlNodeList nl = document.GetElementsByTagName("UML:ActionState");
			
			foreach(XmlNode node in nl)
			{

				if(id.Equals(node.Attributes["xmi.id"].Value) && node.HasChildNodes)
				{
						foreach(XmlNode subnode in node)
						{						
							if(subnode.Name.Equals("UML:ModelElement.stereotype"))
							{							
								string stereotype = subnode.FirstChild.Attributes["xmi.idref"].Value;
					
								XmlNodeList sterotypes = document.GetElementsByTagName("UML:Stereotype");

								foreach(XmlNode subsubnode in sterotypes)
								{
									if(subsubnode.Attributes["xmi.id"] != null)
									{
										if(subsubnode.Attributes["xmi.id"].Value.Equals(stereotype))
										{
											bf.AddStereotype(subsubnode.Attributes["name"].Value);
										}
									}
								}
							}					
					    }
				}
			}
		}
	}
}
