using System;
using System.Collections;
using System.Xml;
using System.Runtime.Serialization;

namespace EnterpriseRepresentation
{
    [Serializable]
	public class SequenceReader
	{
		XmlTextReader reader;
		XmlDocument document = new XmlDocument();
		Environment env;
		System from = null;
		System too = null;
		Hashtable lifelines;
		ArrayList messagesequence = new ArrayList();
		object parent;
		string filename = "";		

		public SequenceReader(){}

		private void GetMessageSequence()
		{
			int num = 1;
			string id = "";

			id = GetMessageIDBySequenceNum(num);
			num++;

			while(id != "")
			{
				messagesequence.Add(id);
				id = GetMessageIDBySequenceNum(num);
				num++;
			}

            if (messagesequence.Count == 0)
            {
                messagesequence = new ArrayList(GetMessageList());
            }
            else
            {
                messagesequence.Reverse();
            }
		}

        private ArrayList GetMessageList()
        {
            ArrayList messages = new ArrayList();

            XmlNodeList nl = document.GetElementsByTagName("UML:Message");
            string id = "";

            foreach (XmlNode node in nl)
            {
                id = node.Attributes["xmi.id"].Value;
                messages.Add(id);
            }

            return messages;
        }

		private string GetMessageIDBySequenceNum(int num)
		{
			XmlNodeList nl = document.GetElementsByTagName("UML:Message");
			string id = "";

			foreach(XmlNode node in nl)
			{						
				id = node.Attributes["xmi.id"].Value;
				
				if(node.HasChildNodes)
				{
					foreach(XmlNode subnode in node)
					{						
						if(subnode.Name.Equals("UML:ModelElement.taggedValue"))
						{
							foreach(XmlNode subsubnode in subnode)
							{
								if(subsubnode.Name.Equals("UML:TaggedValue"))
								{
									if(subsubnode.Attributes["tag"].Value.Equals("seq"))
									{
										if(int.Parse(subsubnode.Attributes["value"].Value) == num)
										{
											return id;
										}
									}
								}
							}
						}
					}					
				}			
			}

			return "";
		}

		public SystemOperation GetSystemOperations(Environment enterprise, string filename, BusinessFunction parent)
		{	
			this.filename = filename;
			reader = new XmlTextReader(filename);
			this.parent = parent;
			document.Load(reader);
			ParseLifeLines();
			GetMessageSequence();
			env = enterprise;
			ArrayList msg = new ArrayList(messagesequence);

			SystemOperation first = CreateSystemOperationMessage(msg[msg.Count-1].ToString());
			msg.RemoveAt(msg.Count-1);
			SystemOperation current;
			SystemOperation newop;
			current = first;

			while(msg.Count>0)
			{
				newop = CreateSystemOperationMessage((string)msg[msg.Count-1]);
				current.next = newop;
				newop.prev = current;
				current = newop;				
				msg.RemoveAt(msg.Count-1);
			}

			reader.Close();
			return first;
		}


		public SubSystemOperation GetSubSystemOperations(System from, System too,string filename, SystemOperation parent)
		{	
			this.from = from;
			this.too = too;
			this.filename = filename;
			reader = new XmlTextReader(filename);
			this.parent = parent;
			document.Load(reader);
			ParseLifeLines();
			GetMessageSequence();

			ArrayList msg = new ArrayList(messagesequence);

            SubSystemOperation first;
			first = CreateSubSystemOperationMessage((string)msg[msg.Count-1]);
			msg.RemoveAt(msg.Count-1);

			SubSystemOperation current;
			SubSystemOperation newop;

			current = first;

			while(msg.Count>0)
			{
				newop = CreateSubSystemOperationMessage((string)msg[msg.Count-1]);
				current.next = newop;
				newop.prev = current;
				current = newop;				
				msg.RemoveAt(msg.Count-1);
			}

			reader.Close();
			return first;
		}

		private SubSystemOperation CreateSubSystemOperationMessage(string id)
		{
			string name = "";
			string reciever = "";
			string sender = "";
			string mid = "";
			string action = "";		

			XmlNodeList nl = document.GetElementsByTagName("UML:Message");			
			
			foreach(XmlNode node in nl)
			{								
				mid = node.Attributes["xmi.id"].Value;
				
				if(mid.Equals(id))
				{
					name = node.Attributes["name"].Value;
					reciever = node.Attributes["receiver"].Value;
					sender = node.Attributes["sender"].Value;
					action = node.Attributes["action"].Value;

					SubSystemOperation subsysop = new SubSystemOperation((SystemOperation)parent,mid);
					subsysop.Name=name;

					subsysop.fromcomponent = GetReferenceToComponent(from.components,lifelines[sender].ToString());
					subsysop.tocomponent = GetReferenceToComponent(too.components,lifelines[reciever].ToString());

                    if (subsysop.tocomponent != null)
                    {
                        subsysop.tocomponent.AddSubSystemOperation(subsysop);
                    }

                    if (subsysop.fromcomponent != null)
                    {
                        subsysop.fromcomponent.AddSubSystemOperation(subsysop);
                    }


					if(node.HasChildNodes)
					{
						foreach(XmlNode subnode in node)
						{						
							if(subnode.Name.Equals("UML:ModelElement.stereotype"))
							{
                                foreach (XmlNode subsubnode in subnode)
                                {
                                    subsysop.AddStereotype(GetStereotypeByID(subsubnode.Attributes["xmi.idref"].Value));
                                }
								
							}
							else if(subnode.Name=="UML:ModelElement.taggedValue")
							{
								foreach(XmlNode subsubnode in subnode)
								{
									string tagname = subsubnode.Attributes["tag"].Value;
									string tagvalue = subsubnode.Attributes["value"].Value;

									subsysop.AddTag(new TagValue(tagname,tagvalue));
								}						
							}
						}					
					}
			
					return subsysop;
				}
			}

			return null;
		}

		private string GetStereotypeByID(string id)
		{
			foreach(XmlNode node in document.GetElementsByTagName("UML:Stereotype"))
			{				
				if(node.Attributes["name"] != null)
				{
					if(node.Attributes["xmi.id"].Value.Equals(id))
				    {
						return node.Attributes["name"].Value;
					}
				}
			}

			return "";
		}

		private SystemOperation CreateSystemOperationMessage(string id)
		{
			string name = "";
			string reciever = "";
			string sender = "";
			string mid = "";
			string action = "";	
	
			XmlNodeList nl = document.GetElementsByTagName("UML:Message");			
			
			foreach(XmlNode node in nl)
			{								
				mid = node.Attributes["xmi.id"].Value;
				
				if(mid.Equals(id))
				{
					name = node.Attributes["name"].Value;
					reciever = node.Attributes["receiver"].Value;
					sender = node.Attributes["sender"].Value;
					action = node.Attributes["action"].Value;
					id = node.Attributes["xmi.id"].Value;

					SystemOperation sysop = new SystemOperation((BusinessFunction)parent,mid);
					sysop.Name=name;
					sysop.fromsystem = GetReferanceToSystem(env,(string)lifelines[sender]);
					sysop.toosystem = GetReferanceToSystem(env,(string)lifelines[reciever]);
					sysop.fromsystem.AddSystemOperation(sysop);
					sysop.toosystem.AddSystemOperation(sysop);
					
					if(node.HasChildNodes)
					{
						foreach(XmlNode subnode in node)
						{						
							if(subnode.Name.Equals("UML:ModelElement.stereotype"))
							{
                                foreach (XmlNode subsubnode in subnode)
                                {
                                    sysop.AddStereotype(GetStereotypeByID(subsubnode.Attributes["xmi.idref"].Value));
                                }
								
							}
							else if(subnode.Name=="UML:ModelElement.taggedValue")
							{
								foreach(XmlNode subsubnode in subnode)
								{
									string tagname = subsubnode.Attributes["tag"].Value;
									string tagvalue = subsubnode.Attributes["value"].Value;

									sysop.AddTag(new TagValue(tagname,tagvalue));
								}						
							}
						}					
					}
			
					return sysop;
				}
			}

			return null;
		}

		private System GetReferanceToSystem(Environment environment,string name)
		{
			foreach(Environment e in environment.environments)
			{
				System temp = GetReferanceToSystem(e,name);

				if(temp != null)
					return temp;
			}
			foreach(System s in environment.systems)
			{
				if(s.name.Equals(name))
				{
					return s;
				}
			}

			return null;
		}

		private Component GetReferenceToComponent(ArrayList comp, string name)
		{
            Component temp = null;

            foreach (Component c in comp)
            {
                temp = GetReferenceToComponentRec(c, name);

                if (temp != null)
                    break;
            }
               
            return temp;
		}

		private Component GetReferenceToComponentRec(Component component,string name)
		{
			
			if(component != null && component.Name.Equals(name))
			{
				return component;
			}

			if(component != null)
			{
				foreach(Component c in component.components)
				{				
					Component newcomp = GetReferenceToComponentRec(c,name);

					if(newcomp != null)
						return newcomp;				
				}
			}

          	return null;
		}

		private void ParseLifeLines()
		{
			string name = "";
			string id = "";
			lifelines = new Hashtable();

			XmlNodeList nl = document.GetElementsByTagName("UML:ClassifierRole");
			
			foreach(XmlNode node in nl)
			{
				name = node.Attributes["name"].Value;				
				id = node.Attributes["xmi.id"].Value;
					
				lifelines.Add(id,name);
			}
		}

	}
}