using System;
using System.Collections;
using System.Xml;

namespace RiskAnalysis
{
	public class DeploymentDiagram : Diagram
	{
		private System.Xml.XmlTextReader reader;
		string filename = "";
		public ArrayList components = new ArrayList();
		public ArrayList nodes = new ArrayList();
        public ArrayList assoications = new ArrayList();
        public ArrayList dependencies = new ArrayList();

		public DeploymentDiagram(string filename)
		{
			this.filename = filename;
			ParseStereotypes(filename);
			ParseNodes();
			ParseComponents();
			ParseAssociations();
			ParseDependencies();
		}

		public void ParseComponents()
		{
			components = new ArrayList();
			string name = "";
			string id = "";

			reader = new System.Xml.XmlTextReader(filename);
			XmlDocument document = new XmlDocument();
			document.Load(reader);

			XmlNodeList nl = document.GetElementsByTagName("UML:Component");
			
			foreach(XmlNode node in nl)
			{

				if(!node.ParentNode.Name.Equals("UML:AssociationEnd.participant"))
				{
                    if (node.Attributes["name"] != null)
                    {
                        name = node.Attributes["name"].Value;
                        id = node.Attributes["xmi.id"].Value;

                        Component c = new Component(name, id, new ArrayList());

                        if (node.HasChildNodes)
                        {
                            ExtractStereotypeAndTags(node, c);    
                        }

                          components.Add(c);
                    }

					

				}
				
			}

		}


		public void ParseDependencies()
		{
			string id = "";
			//string stereotype = "";
			string from = "";
			string to = "";
			dependencies = new ArrayList();

			reader = new System.Xml.XmlTextReader(filename);
			XmlDocument document = new XmlDocument();
			document.Load(reader);

			XmlNodeList nl = document.GetElementsByTagName("UML:Dependency");
			
			foreach(XmlNode node in nl)
			{

				//name = node.Attributes["name"].Value;				
				id = node.Attributes["xmi.id"].Value;
				from = node.Attributes["supplier"].Value;
				to = node.Attributes["client"].Value;
                ComponentDependency cd = new ComponentDependency(id, new ArrayList(), GetObjectByID(from), GetObjectByID(to));
			

				if(node.HasChildNodes)
				{
                    ExtractStereotypeAndTags(node, cd);		
				}

				dependencies.Add(cd);

			}

		}


		public object GetObjectByID(string id)
		{
			foreach(Component comp in components)
			{
				if(comp.id.Equals(id))
				{
					return comp;
				}
			}

			foreach(Node node in nodes)
			{
				if(node.id.Equals(id))
				{
					return node;
				}
			}

			return null;
		}

		public void ParseAssociations()
		{ 
			string id = "";
			assoications = new ArrayList();

			reader = new System.Xml.XmlTextReader(filename);
			XmlDocument document = new XmlDocument();
			document.Load(reader);

			XmlNodeList nl = document.GetElementsByTagName("UML:Association");
			
			foreach(XmlNode node in nl)
			{

				id = node.Attributes["xmi.id"].Value;
                Association ass = new Association(id, new ArrayList(), null,null);
				
				if(node.HasChildNodes)
				{
                    ExtractStereotypeAndTags(node, ass);

					foreach(XmlNode subnode in node)
					{						
						if(subnode.Name=="UML:Association.connection")
						{
							foreach(XmlNode subsubnode in subnode.ChildNodes)
							{
								if(subsubnode.Name.Equals("UML:AssociationEnd") && ass.to == null)
								{
                                    ass.to = GetObjectByID(subsubnode.Attributes["participant"].Value);
								}
								else if(subsubnode.Name.Equals("UML:AssociationEnd") && ass.from == null)
								{
                                    ass.from = GetObjectByID(subsubnode.Attributes["participant"].Value);
								}                                
							}
						}
					}					
				}				
				assoications.Add(ass);
			}


		}


		public void ParseNodes()
		{
			string name = "";
			string id = "";
			nodes = new ArrayList();

			reader = new System.Xml.XmlTextReader(filename);
			XmlDocument document = new XmlDocument();
			document.Load(reader);

			XmlNodeList nl = document.GetElementsByTagName("UML:Node");
			
			foreach(XmlNode node in nl)
			{
				name = node.Attributes["name"].Value;				
				id = node.Attributes["xmi.id"].Value;
                Node n = new Node(name, id, new ArrayList());

				if(node.HasChildNodes)
				{
                    ExtractStereotypeAndTags(node, n);				
				}

				nodes.Add(n);

			}

		}

	}
}
