using System;
using System.Collections;
using System.Text;
using System.Xml;

namespace RiskAnalysis
{
    public class Diagram
    {
        ArrayList stereotypes;

        public void ExtractStereotypeAndTags(XmlNode node, DiagramElement de)
        {
            foreach (XmlNode subnode in node)
            {
                if ((subnode.Name.Equals("UML:ModelElement.stereotype") && (subnode.FirstChild.Attributes["xmi.idref"] != null)))
                {
                    de.AddStereotype(GetStereotypeByID(subnode.FirstChild.Attributes["xmi.idref"].Value));
                }
                else if (subnode.Name.Equals("UML:ModelElement.taggedValue"))
                {

                    foreach (XmlNode subsubnode in subnode)
                    {
                        de.AddTag(new TagValue(subsubnode.Attributes["tag"].Value, subsubnode.Attributes["value"].Value));
                    }
                }
            }
        }

        public Stereotype GetStereotypeByID(string id)
        {
            if (stereotypes != null)
            {
                foreach (Stereotype s in stereotypes)
                {
                    if (s.id.Equals(id))
                        return s;
                }
            }

            return null;
        }



        public void ParseStereotypes(string filename)
        {
            XmlTextReader reader = new System.Xml.XmlTextReader(filename);
            XmlDocument document = new XmlDocument();
            document.Load(reader);

            string id = "";
            string name = "";
            stereotypes = new ArrayList();

            XmlNodeList nl = document.GetElementsByTagName("UML:Stereotype");

            foreach (XmlNode node in nl)
            {
                if (node.Attributes["xmi.id"] != null)
                {
                    id = node.Attributes["xmi.id"].Value;
                    name = node.Attributes["name"].Value;
                    stereotypes.Add(new Stereotype(id, name));
                }
            }
        }
    }
}
