using System;
using System.Collections;
using System.Xml;
using System.Data;

namespace RiskAnalysis
{
	public class HazopTransformer
	{
		
		HazopTransformationProfileManager config = new HazopTransformationProfileManager();
		
		public HazopTransformer(){}

		public ArrayList ParseSequenceDiagram(int profile, string filename)
     	{
			ArrayList output = new ArrayList();
			SequenceDiagram sd = new SequenceDiagram(filename);
			DataTable studynodes = config.GetStudyNodes(profile);
			DataTable parameters;
			DataTable guidewords;
			string parameter = "";
			string name = "";
			string guideword = "";
			string type = "";

			for(int i=0;i<studynodes.Rows.Count;i++)
			{
				type = (string)studynodes.Rows[i][1];            			

				foreach(SequenceMessage message in sd.GetMessages())
				{
                    foreach (Stereotype s in message.Stereotypes)
                    {
                        if (s.name.Equals(type))
                        {
                            name = message.Name;
                            parameters = config.GetParametersByName(type);

                            for (int j = 0; j < parameters.Rows.Count; j++)
                            {
                                guidewords = config.GetGuideWords((int)parameters.Rows[j][0]);
                                parameter = (string)parameters.Rows[j][1];

                                for (int k = 0; k < guidewords.Rows.Count; k++)
                                {
                                    guideword = (string)guidewords.Rows[k][1];
                                    output.Add(new HazopOutputItem(s.name, message.Name + " From:" + message.FromLifeLine() + " To: " + message.ToLifeLine(), parameter, guideword));

                                }
                            }
                        }
                    }
				}

                foreach (SequenceLifeLine lifeline in sd.GetLifeLines())
                {
                    foreach (Stereotype s in lifeline.Stereotypes)
                    {
                        if (s.name.Equals(type))
                        {
                            name = lifeline.Name;
                            parameters = config.GetParametersByName(type);

                            for (int j = 0; j < parameters.Rows.Count; j++)
                            {
                                guidewords = config.GetGuideWords((int)parameters.Rows[j][0]);
                                parameter = (string)parameters.Rows[j][1];

                                for (int k = 0; k < guidewords.Rows.Count; k++)
                                {
                                    guideword = (string)guidewords.Rows[k][1];
                                    output.Add(new HazopOutputItem(s.name, lifeline.Name, parameter, guideword));

                                }
                            }
                        }
                    }
                }
			}

			return output;

        }

		public ArrayList ParseActivityDiagram(int profile,string filename)
		{

			ArrayList output = new ArrayList();

			//string filename = "";
			string type = "";

			ActivityDiagram diagram = new ActivityDiagram(filename);
			DataTable studynodes = config.GetStudyNodes(profile);
			DataTable parameters;
			DataTable guidewords;

			for(int i=0;i<studynodes.Rows.Count;i++)
			{
				type = (string)studynodes.Rows[i][1];            			

				foreach(ActivityState state in diagram.actionstates)
				{
                    foreach (Stereotype s in state.stereotypes)
                    {
                        if (s != null && s.name.Equals(type))
                        {
                            string name = state.name;

                            parameters = config.GetParametersByName(type);

                            for (int j = 0; j < parameters.Rows.Count; j++)
                            {
                                guidewords = config.GetGuideWords((int)parameters.Rows[j][0]);
                                string parameter = (string)parameters.Rows[j][1];

                                for (int k = 0; k < guidewords.Rows.Count; k++)
                                {
                                    string guideword = (string)guidewords.Rows[k][1];

                                    output.Add(new HazopOutputItem(type, name, parameter, guideword));
                                }
                            }
                        }
                    }
				}

                foreach (ActivityTransition transition in diagram.transitions)
                {
                    foreach (Stereotype s in transition.stereotypes)
                    {
                        if (s != null && s.name.Equals(type))
                        {
                            string name = transition.source.Name + " - "+ transition.target.Name;

                            parameters = config.GetParametersByName(type);

                            for (int j = 0; j < parameters.Rows.Count; j++)
                            {
                                guidewords = config.GetGuideWords((int)parameters.Rows[j][0]);
                                string parameter = (string)parameters.Rows[j][1];

                                for (int k = 0; k < guidewords.Rows.Count; k++)
                                {
                                    string guideword = (string)guidewords.Rows[k][1];

                                    output.Add(new HazopOutputItem(type, name, parameter, guideword));
                                }
                            }
                        }
                    }
                }
			}

			return output;

		}       

		
	}	
}
