using System;
using System.Data.OleDb;
using System.Data;

namespace RiskAnalysis
{
	/// <summary>
	/// Summary description for PHAConfigurationHandler.
	/// </summary>
	public class PHATransformationProfileManager
	{
		//Class members
		OleDbConnection conn = new OleDbConnection();
		OleDbDataAdapter adapter = new OleDbDataAdapter();
		OleDbCommand command = new OleDbCommand();
		string StartUpPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);
		String sql = "";
	
		public PHATransformationProfileManager()
		{
			SetUpDatabaseConnection();
		}

		public void RemoveProfile(int id)
		{

			//Deletes the Profile
			RemoveSingleRow("CheckListProfile",id);
			
			conn.Open();
						
			//Deletes all the parameters connected with the profile
			sql = "DELETE FROM CheckListItem WHERE CheckListItem.Studynode IN ";
			sql += "(SELECT ID FROM StudyNode WHERE Profile = " + id + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		
			//Deletes all the study nodes connected with a profile.
			RemoveStudyNodeByProfile(id);


		}

		public void SetUpDatabaseConnection()
		{
			//Sets the connection string
			conn.ConnectionString=@"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + StartUpPath + "\\phaconfig.mdb";
		}

		public DataTable GetDiagramTypes()
		{
			return GetDataTableFromDatabase("SELECT ID,NAME FROM DiagramType");
		}

		public DataTable GetProfiles()
		{			
			return GetDataTableFromDatabase("SELECT CheckListProfile.ID,CheckListProfile.Name,DiagramType.Name,DiagramType.ID FROM CheckListProfile LEFT JOIN DiagramType ON DiagramType.ID = CheckListProfile.DiagramType");			
		}

        public DataTable GetProfilesByID(string id)
        {
            return GetDataTableFromDatabase("SELECT ID, Name, DiagramType FROM CheckListProfile WHERE DiagramType =" + id); 
      
        }

		public void AddProfile(string name, int diagramtype)
		{
			conn.Open();
			sql = "INSERT INTO CheckListProfile(Name, DiagramType) VALUES('" + name + "',"  + diagramtype + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

		public DataTable GetUMLEntityTypes(int diagramtype)
		{
			return GetDataTableFromDatabase("SELECT ID,TypeName FROM UMLEntityType WHERE DiagramType =" + diagramtype);
		}

		public DataTable GetStudyNodes(int profile)
		{			
			return GetDataTableFromDatabase("SELECT StudyNode.ID,StudyNode.Name,UMLEntityType.TypeName FROM StudyNode, UMLEntityType WHERE UMLEntityType.ID = StudyNode.ExtendsEntity AND StudyNode.Profile = " + profile);		
		}


		public void AddCheckListItem(string name, string desc, int studynode)
		{			
			conn.Open();
			sql = "INSERT INTO CheckListItem(Name, StudyNode, ListItemText) VALUES('" + name + "',"  + studynode + ",'" + desc + "')";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}


        public void AddTagCheckListItem(string name, int studynode,string desc, string tag, string op, string value)
        {

            conn.Open();
            sql = "INSERT INTO TagCheckListItem(Name, StudyNode, ListItemText, TagName, Operator, TagValue) VALUES('" + name + "'," + studynode + ",'" + desc + "'" + ",'" + tag + "'" + ",'" + op + "','" + value + "')";
            command = new OleDbCommand(sql, conn);
            command.ExecuteNonQuery();
            conn.Close();

        }







        public void RemoveTagCheckListItem(int id)
		{
			RemoveMultipleRows("TagCheckListItem","ID",id);
		}

		public void RemoveCheckListItemsByStudyNode(int id)
		{
			RemoveMultipleRows("CheckListItem","StudyNode",id);
		}

		public void RemoveStudyNode(int id)
		{

			RemoveSingleRow("StudyNode",id);
			RemoveCheckListItemsByStudyNode(id);

		}

		public DataTable GetCheckListItems(int studynode)
		{
			return GetDataTableFromDatabase("SELECT ID, Name, ListItemText FROM CheckListItem WHERE Studynode =" + studynode);
		}

        public DataTable GetTagCheckListItems(int studynode)
		{
			return GetDataTableFromDatabase("SELECT ID, Name, ListItemText, TagName, TagValue, Operator FROM TagCheckListItem WHERE Studynode =" + studynode);
		}

        



		private void RemoveSingleRow(string table,int id)
		{
			conn.Open();
			sql = "DELETE FROM " + table + " WHERE ID=" + id;
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

		public void RemoveStudyNodeByProfile(int id)
		{
			RemoveMultipleRows("Studynode","Profile",id);
		}

		private void RemoveMultipleRows(string table, string attributt, int id)
		{
			conn.Open();
			sql = "DELETE FROM " + table + " WHERE " + attributt + "=" + id;
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

		public void RemoveCheckListItem(int id)
		{
			RemoveSingleRow("CheckListItem",id);
		}


		public void AddStudyNode(string name,int profile, int extends)
		{
			conn.Open();
			sql = "INSERT INTO StudyNode(Name, Profile, ExtendsEntity) VALUES('" + name + "',"  + profile + "," + extends + ")";
			command = new OleDbCommand(sql,conn);
			command.ExecuteNonQuery();
			conn.Close();
		}

		public DataTable GetCheckListItemsByType(string studynodename)
		{
			return GetDataTableFromDatabase("SELECT CheckListItem.ID, CheckListItem.Name, CheckListItem.ListItemText, StudyNode.Name FROM CheckListItem, StudyNode WHERE CheckListItem.StudyNode = StudyNode.ID AND StudyNode.Name ='" + studynodename + "'");
		}

        public DataTable GetTagCheckListItemsByType(string studynodename)
        {
            return GetDataTableFromDatabase("SELECT TagCheckListItem.ID, TagCheckListItem.Name, TagCheckListItem.ListItemText, StudyNode.Name, TagCheckListItem.TagName, TagCheckListItem.Operator, TagCheckListItem.TagValue FROM TagCheckListItem, StudyNode WHERE TagCheckListItem.StudyNode = StudyNode.ID AND StudyNode.Name ='" + studynodename + "'");
        }

		private DataTable GetDataTableFromDatabase(string sql)
		{
			conn.Open();						
			DataSet data = new DataSet();			
			adapter = new OleDbDataAdapter(sql,conn);
            try
            {
                adapter.Fill(data);
            }
            catch{

                conn.Close();
                return null;
            }

            conn.Close();
			return data.Tables[0];
		}

	}
}
