using System;
using System.Data;
using System.Collections;

namespace RiskAnalysis
{
	public class PHATransformer
	{

		PHATransformationProfileManager config = new PHATransformationProfileManager();
		public PHATransformer(){}

        public ArrayList GeneratePHAOutputItems(string type, string name, DiagramElement de)
        {
            ArrayList output = new ArrayList();

            foreach(Stereotype s in de.stereotypes)
            {
                if (s != null && s.Name.Equals(type))
                {
                    DataTable checklistitems = config.GetCheckListItemsByType(type);
                    DataTable tagchecklistitems = config.GetTagCheckListItemsByType(type);

                    for (int i = 0; i < checklistitems.Rows.Count; i++)
                    {
                        string checlistitemname = (string)checklistitems.Rows[i][1];
                        string checklistdesc = (string)checklistitems.Rows[i][2];
                        output.Add(new PHAOutputItem(name, checlistitemname, checklistdesc));
                    }

                    for (int k = 0; k < tagchecklistitems.Rows.Count; k++)
                    {

                        string op = (string)tagchecklistitems.Rows[k][5];
                        string tagname = (string)tagchecklistitems.Rows[k][4];
                        string tagvalue = (string)tagchecklistitems.Rows[k][6];
                        string checlistitemname = (string)tagchecklistitems.Rows[k][1];
                        string checklistdesc = (string)tagchecklistitems.Rows[k][2];
                        bool correct = false;

                        if (de.GotTagName(tagname))
                        {
                            if (op.Equals("="))
                            {
                                if (de.GetTagValue(tagname).Equals(tagvalue))
                                {
                                    correct = true;
                                }
                            }
                            else if (op.Equals("!="))
                            {
                                if (!de.GetTagValue(tagname).Equals(tagvalue))
                                {

                                    correct = true;
                                }
                            }
                            else if (op.Equals("<"))
                            {
                                string value = de.GetTagValue(tagname);
                                double a;
                                double b;

                                if (double.TryParse(value, out a) && double.TryParse(tagvalue, out b))
                                {

                                    if (a < b)
                                    {
                                        correct = true;
                                    }
                                }

                            }
                            else if (op.Equals(">"))
                            {
                                string value = de.GetTagValue(tagname);
                                double a, b;

                                if (double.TryParse(value, out a) && double.TryParse(tagvalue, out b))
                                {
                                    if (a > b)
                                    {
                                        correct = true;
                                    }
                                }
                            }

                            if (correct)
                            {
                                output.Add(new PHAOutputItem(tagname + op + tagvalue, checlistitemname, checklistdesc));
                            }
                        }
                    }
                }
            }       

                return output;            
        }

		public ArrayList ParseComponentDiagram(int profile,string filename)
		{
			ArrayList output = new ArrayList();
			string type = "";
			string name = "";
			ComponentDiagram diagram = new ComponentDiagram(filename);
			DataTable studynodes = config.GetStudyNodes(profile);

            for (int i = 0; i < studynodes.Rows.Count; i++)
            {
                type = (string)studynodes.Rows[i][1];

                foreach (Component comp in diagram.components)
                {
                    output.AddRange(GeneratePHAOutputItems(type, comp.name, comp));
                }

                foreach (Association ass in diagram.assoications)
                {
                    foreach (Stereotype s in ass.stereotypes)
                    {
                        if (s.name.Equals(type))
                        {
                            if (ass.from.GetType().Equals("Component"))
                            {
                                name = "From: " + ((Component)(ass.from)).name;
                            }
                            else if (ass.from.GetType().Equals("Node"))
                            {
                                name = "From: " + ((Node)(ass.from)).name;
                            }

                            if (ass.to.GetType().Equals("Component"))
                            {
                                name += " To: " + ((Component)(ass.to)).name;
                            }
                            else if (ass.to.GetType().Equals("Node"))
                            {
                                name += " To: " + ((Node)(ass.to)).name;
                            }

                            output.AddRange(GeneratePHAOutputItems(type, name, ass));

                        }
                    }
                }

                foreach (ComponentDependency depend in diagram.dependencies)
                {
                    foreach (Stereotype s in depend.stereotypes)
                    {
                        if (s.Name.Equals(type))
                        {
                            name = "";

                            if (depend.from.GetType().ToString().Equals("RiskAnalysis.Component"))
                            {
                                name = type + " From: " + ((Component)(depend.from)).name;
                            }

                            if (depend.to.GetType().ToString().Equals("RiskAnalysis.Component"))
                            {
                                name += " To: " + ((Component)(depend.to)).name;
                            }

                            output.AddRange(GeneratePHAOutputItems(type, name, depend));
                        }
                    }
                }                
            }

            return output;
		
		}

		public ArrayList ParseDeploymentDiagram(int profile, string filename)
		{

			ArrayList output = new ArrayList();
			string type = "";
			string name = "";

			DeploymentDiagram diagram = new DeploymentDiagram(filename);
			DataTable studynodes = config.GetStudyNodes(profile);

            for (int i = 0; i < studynodes.Rows.Count; i++)
            {
                type = (string)studynodes.Rows[i][1];

                foreach (Node node in diagram.nodes)
                {
                    output.AddRange(GeneratePHAOutputItems(type, node.name, node));
                }

                foreach (Component comp in diagram.components)
                {
                    output.AddRange(GeneratePHAOutputItems(type, comp.name, comp));
                }

                foreach (Association ass in diagram.assoications)
                {
                    foreach (Stereotype s in ass.stereotypes)
                    {
                        if (s.name.Equals(type))
                        {
                            name = "" + type;

                            if (ass.from.GetType().ToString().Equals("RiskAnalysis.Component"))
                            {
                                name = type + " From: " + ((Component)(ass.from)).name;
                            }
                            else if (ass.from.GetType().ToString().Equals("RiskAnalysis.Node"))
                            {
                                name = "From: " + ((Node)(ass.from)).name;
                            }

                            if (ass.to.GetType().ToString().Equals("RiskAnalysis.Component"))
                            {
                                name += " To: " + ((Component)(ass.to)).name;
                            }
                            else if (ass.to.GetType().ToString().Equals("RiskAnalysis.Node"))
                            {
                                name += " To: " + ((Node)(ass.to)).name;
                            }

                            output.AddRange(GeneratePHAOutputItems(type, name, ass));
                        }
                    }
                }

                foreach (ComponentDependency depend in diagram.dependencies)
                {
                    foreach (Stereotype s in depend.stereotypes)
                    {
                        if (s.name.Equals(type))
                        {

                            if (depend.from.GetType().ToString().Equals("Component"))
                            {
                                name = type + " From: " + ((Component)(depend.from)).name;
                            }
                            else if (depend.from.GetType().ToString().Equals("Node"))
                            {
                                name = "From: " + ((Node)(depend.from)).name;
                            }

                            if (depend.to.GetType().ToString().Equals("Component"))
                            {
                                name += " To: " + ((Component)(depend.to)).name;
                            }
                            else if (depend.to.GetType().ToString().Equals("Node"))
                            {
                                name += " To: " + ((Node)(depend.to)).name;
                            }

                            output.AddRange(GeneratePHAOutputItems(type, name, depend));
                        }
                    }
                }
            }

			return output;

		}

		public ArrayList ParseActivityDiagram(int profile,string filename)
		{

			ArrayList output = new ArrayList();
			string type = "";

			ActivityDiagram diagram = new ActivityDiagram(filename);
			DataTable studynodes = config.GetStudyNodes(profile);

			for(int i=0;i<studynodes.Rows.Count;i++)
			{
				type = (string)studynodes.Rows[i][1];									

				foreach(ActivityState state in diagram.actionstates)
				{
                    output.AddRange(GeneratePHAOutputItems(type, state.name, state));
				}

                foreach(ActivityTransition trans in diagram.transitions)
                {
                   string name = "Transition from " + trans.source.Name + " to " + trans.target.name;
                   output.AddRange(GeneratePHAOutputItems(type, name, trans));
                }
			}

			return output;

		}    
    }
}
