using System;
using System.Xml;
using System.Collections;

namespace RiskAnalysis
{	
	public class SequenceDiagram : Diagram
	{
		private string filename = "";
		private XmlTextReader reader;

		ArrayList messages;
		ArrayList lifelines;

		public SequenceDiagram(string filename)
		{
			this.filename = filename;
			
			ParseStereotypes(filename);
            ParseLifeLines();
			ParseMessages();        
			
		}

		public ArrayList GetMessages(){return messages;}
		public ArrayList GetLifeLines(){return lifelines;}

		private SequenceLifeLine GetLifeLineByID(string id)
		{
			foreach(SequenceLifeLine lifeline in lifelines)
			{
				if(lifeline.ID.Equals(id))
				{
					return lifeline;
				}
			}

			return null;
		}

		private void ParseLifeLines()
		{

			string name = "";
			string id = "";
			lifelines= new ArrayList();

			reader = new System.Xml.XmlTextReader(filename);
			XmlDocument document = new XmlDocument();
			document.Load(reader);

			XmlNodeList nl = document.GetElementsByTagName("UML:ClassifierRole");
			
			foreach(XmlNode node in nl)
			{

				name = node.Attributes["name"].Value;				
				id = node.Attributes["xmi.id"].Value;
                SequenceLifeLine sl = new SequenceLifeLine(name,id,new ArrayList());
				
				if(node.HasChildNodes)
				{
                    ExtractStereotypeAndTags(node, sl);					
				}

                lifelines.Add(sl);
			}
		}



		private void ParseMessages()
		{
			reader = new System.Xml.XmlTextReader(filename);
			string name = "";
			string reciever = "";
			string sender = "";
			string id = "";
			string action = "";
			messages = new ArrayList();
			XmlDocument document = new XmlDocument();
			document.Load(reader);
			XmlNodeList nl = document.GetElementsByTagName("UML:Message");
			
			foreach(XmlNode node in nl)
			{
				name = node.Attributes["name"].Value;				
				id = node.Attributes["xmi.id"].Value;
				reciever = node.Attributes["receiver"].Value;
				sender = node.Attributes["sender"].Value;
				action = node.Attributes["action"].Value;
                
                SequenceMessage sm = new SequenceMessage(name,GetLifeLineByID(sender),GetLifeLineByID(reciever),id);

				if(node.HasChildNodes)
				{
                    ExtractStereotypeAndTags(node,sm);									
				}
					
				messages.Add(sm);
            }

		}
		
	}
}
